Package: rtemis.a3
Title: Amino Acid Annotation (A3) Format
Version: 0.5.3
Date: 2026-04-24
Authors@R: 
    person(given = "E.D.", family = "Gennatas", role = c("aut", "cre", "cph"),
           email = "gennatas@gmail.com",
           comment = c(ORCID = "0000-0001-9280-3609"))
Description: Implements the Amino Acid Annotation (A3) format using 'S7' classes.
    The A3 format is a structured schema for annotating amino acid sequences with site, region, 
    post-translational modification (PTM), processing event, and variant information. 
    Provides functions to create, read, and write A3 objects, and to import annotations from 
    'UniProt', 'AlphaFold', 'PDBe', 'ClinVar', and 'Ensembl'.
URL: https://a3.rtemis.org
BugReports: https://github.com/rtemis-org/a3/issues
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: cli, data.table, rtemis.core, S7, utils
Suggests: biomaRt, httr, jsonlite, seqinr, testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-25 04:43:12 UTC; sdg
Author: E.D. Gennatas [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-9280-3609>)
Maintainer: E.D. Gennatas <gennatas@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-28 19:30:18 UTC
Built: R 4.4.3; ; 2026-05-01 13:34:25 UTC; unix

# Generated by roxygen2: do not edit by hand

export(aa_sub)
export(annotation_position)
export(annotation_range)
export(annotation_variant)
export(clinvar_variants)
export(concat)
export(create_A3)
export(gene2sequence)
export(get_alphafold)
export(pdb_annotations)
export(read_A3json)
export(uniprot_sequence)
export(uniprot_to_A3)
export(write_A3json)
import(S7)
import(cli)
import(data.table)
import(rtemis.core)
import(utils)
importFrom(grDevices,col2rgb)
importFrom(stats,ave)
importFrom(stats,na.exclude)

[![rtemis.a3 status badge](https://rtemis-org.r-universe.dev/rtemis.a3/badges/version)](https://rtemis-org.r-universe.dev/rtemis.a3)
[![r-ci](https://github.com/rtemis-org/a3/actions/workflows/r-ci.yml/badge.svg)](https://github.com/rtemis-org/a3/actions/workflows/r-ci.yml)

# rtemis.a3

R implementation of the **Amino Acid Annotation (A3)** format —
a structured JSON format for amino acid sequences with site, region, PTM,
processing, and variant annotations.

Part of the [rtemis-org/a3](https://github.com/rtemis-org/a3) monorepo,
which provides A3 implementations in R, Python, Julia, TypeScript, and Rust.

## Installation

### R-universe

```r
pak::repo_add(myuniverse = "https://rtemis-org.r-universe.dev")
pak::pak("rtemis.a3")
```

### GitHub

```r
pak::pak("rtemis-org/a3/r")
```

## Quick Start

```r
library(rtemis.a3)

a3 <- create_A3(
  sequence = "MKTAYIAKQR",
  site = list(
      "Active site" = annotation_position(c(3, 5),
    ),
  region = list(
    "Repeat 1" = annotation_range(rbind(c(1L, 4L)))
  ),
  ptm = list(
    Phosphorylation = annotation_position(c(7))
  ),
    uniprot_id  = "P12345",
    description = "Example protein",
    organism    = "Homo sapiens"
  )
)
```

## Read / Write JSON

```r
write_A3json(a3, "path/to/protein.json")
a3 <- read_A3json("path/to/protein.json")
```

## API

### Construction

|                                      Function | Description                                |
|-----------------------------------------------|--------------------------------------------|
| `create_A3(sequence, site, region, ptm, ...)` | Create an A3 object                        |
|                `annotation_position(x, type)` | Create a position-indexed annotation entry |
|                   `annotation_range(x, type)` | Create a range-indexed annotation entry    |
|                 `annotation_variant(x, info)` | Create a variant annotation                |

### I/O

|                Function | Description                        |
|-------------------------|------------------------------------|
| `write_A3json(x, path)` | Write an A3 object to a JSON file  |
|     `read_A3json(path)` | Read an A3 object from a JSON file |

### S7 Class Hierarchy

```
A3
 ├── sequence:     A3Sequence
 ├── annotations:  A3Annotation
 │   ├── site:        named list of A3Site       (A3Position index)
 │   ├── region:      named list of A3Region     (A3Range index)
 │   ├── ptm:         named list of A3PTM        (A3Index — position or range)
 │   ├── processing:  named list of A3Processing (A3Index — position or range)
 │   └── variant:     list of A3Variant
 └── metadata:     A3Metadata
     ├── uniprot_id
     ├── description
     ├── reference
     └── organism
```

#' Get AlphaFold info for a given UniProt ID
#'
#' @param uniprotid Character: UniProt ID.
#'
#' @return data frame with AlphaFold info.
#'
#' @author EDG
#' @export
#'
#' @examples
#' # Requires internet connection and fetches data from AlphaFold.
#' \dontrun{
#' get_alphafold("P10636")
#' }
get_alphafold <- function(uniprotid) {
  check_scalar_character(uniprotid, arg_name = "uniprotid")
  check_dependencies(c("httr", "jsonlite"))
  url <- paste0("https://www.alphafold.ebi.ac.uk/api/prediction/", uniprotid)
  headers <- c(
    "accept" = "application/json"
  )
  response <- httr::GET(url, httr::add_headers(.headers = headers))
  httr::stop_for_status(response)
  content <- httr::content(response, as = "text", encoding = "UTF-8")
  jsonlite::fromJSON(content)
}


# %% get_alphafold_pdb ----
get_alphafold_pdb <- function(uniprotid) {
  get_alphafold(uniprotid)[["pdb"]]
}


#' Perform amino acid substitutions
#'
#' @param x Character vector: Amino acid sequence. e.g. `"ARND"` or
#' `c("A", "R", "N", "D")`.
#' @param substitutions Character vector: Substitutions to perform in the format
#' "OriginalPositionNew", e.g. `c("C291A", "C322A")`.
#' @param verbosity Integer: Verbosity level.
#'
#' @return Character vector with substitutions performed.
#'
#' @author EDG
#' @export
#'
#' @examples
#' aa_sub(c("A", "R", "N", "D"), c("R2K", "N3S"))
aa_sub <- function(x, substitutions, verbosity = 1L) {
  check_inherits(x, "character")
  check_inherits(substitutions, "character")
  # Split x into characters
  if (length(x) == 1) {
    x <- unlist(strsplit(x, ""))
  }
  for (s in substitutions) {
    strngs <- strsplit(s, "")[[1]]
    from <- strngs[1]
    to <- strngs[length(strngs)]
    pos <- as.numeric(strngs[2:(length(strngs) - 1)] |> paste(collapse = ""))
    msg(
      "Substituting",
      highlight(from),
      "at position",
      highlight(pos),
      "with",
      highlight(to),
      verbosity = verbosity
    )
    x[pos] <- to
  }
  msg("All done.", verbosity = verbosity)
  x
}

# ::rtemis.a3::
# rtemis.org

# %% gene2sequence ----
test_that("gene2sequence works", {
  skip("gene2sequence queries ensembl.org. Avoid running repeatedly.")
  expect_s3_class(
    gene2sequence("MAPT"),
    "data.frame"
  )
})

# %% aa_sub ----
test_that("aa_sub works", {
  expect_equal(
    aa_sub("ACDEFG", c("C2A")),
    c("A", "A", "D", "E", "F", "G")
  )
})

# %% uniprot_sequence ----
#' Fetch a protein sequence from UniProt
#'
#' Lightweight FASTA-only fetch. Use this when you only need the amino acid
#' sequence. For full annotations and metadata use [uniprot_to_A3()].
#'
#' @param accession Character scalar: UniProt accession, e.g. `"P10636"`.
#' @param base_url Character scalar: UniProt REST API base URL.
#' @param verbosity Integer scalar: Verbosity level.
#'
#' @return Character scalar: amino acid sequence in single-letter code.
#'
#' @author EDG
#' @export
#'
#' @examples
#' # Requires internet connection and fetches data from UniProt.
#' \dontrun{
#'   seq <- uniprot_sequence("P10636")
#' }
uniprot_sequence <- function(
  accession,
  base_url = "https://rest.uniprot.org/uniprotkb",
  verbosity = 1L
) {
  check_scalar_character(accession, arg_name = "accession")
  check_scalar_character(base_url, arg_name = "base_url")
  check_dependencies("seqinr")

  path <- paste0(base_url, "/", accession, ".fasta")
  dat <- seqinr::read.fasta(path, seqtype = "AA")
  msg("Got:", highlight(attr(dat[[1L]], "Annot")), verbosity = verbosity)
  paste(as.character(dat[[1L]]), collapse = "")
}


# %% uniprot_to_A3 ----
#' Fetch a UniProt entry and parse it into an A3 object
#'
#' Fetches the UniProt JSON for the given accession via the UniProt REST API,
#' maps all sequence annotations to the A3 schema (see `specs/uniprot.md`), and
#' returns a fully populated `A3` object.
#'
#' Feature types not mapped to A3 (mutagenesis, alternative sequences, sequence
#' conflicts, etc.) are silently ignored.
#'
#' @param accession Character scalar: UniProt accession, e.g. `"P10636"`.
#' @param base_url Character scalar: UniProt REST API base URL.
#' @param verbosity Integer scalar: Verbosity level.
#'
#' @return `A3` object with sequence, annotations, and UniProt metadata.
#'
#' @author EDG
#' @export
#'
#' @examples
#' # Requires internet connection and fetches data from UniProt.
#' \dontrun{
#' mapt <- uniprot_to_A3("P10636")
#' }
uniprot_to_A3 <- function(
  accession,
  base_url = "https://rest.uniprot.org/uniprotkb",
  verbosity = 1L
) {
  check_scalar_character(accession, arg_name = "accession")
  check_scalar_character(base_url, arg_name = "base_url")
  check_dependencies(c("httr", "jsonlite"))

  # -- Fetch ----
  url <- paste0(base_url, "/", accession, ".json")
  response <- httr::GET(url)
  httr::stop_for_status(response)
  dat <- jsonlite::fromJSON(
    httr::content(response, as = "text", encoding = "UTF-8"),
    simplifyVector = FALSE
  )

  # -- Sequence ----
  sequence <- dat[["sequence"]][["value"]]

  # -- Metadata ----
  null_or <- function(x, default = "") if (is.null(x)) default else x

  uniprot_id <- null_or(dat[["primaryAccession"]])
  reference <- null_or(dat[["uniProtkbId"]])
  organism <- null_or(dat[["organism"]][["scientificName"]])

  pd <- dat[["proteinDescription"]]
  description <- if (!is.null(pd)) {
    rn <- pd[["recommendedName"]]
    if (!is.null(rn)) {
      null_or(rn[["fullName"]][["value"]])
    } else {
      sn <- pd[["submittedNames"]]
      if (!is.null(sn) && length(sn) > 0L) {
        null_or(sn[[1L]][["fullName"]][["value"]])
      } else {
        ""
      }
    }
  } else {
    ""
  }

  # -- Feature type → A3 type lookup tables ----
  site_types <- c(
    "Active site" = "activeSite",
    "Binding site" = "bindingSite",
    "Site" = "site",
    "Non-standard residue" = "nonStandardResidue"
  )
  region_types <- c(
    "Domain" = "domain",
    "Repeat" = "repeat",
    "Zinc finger" = "zincFinger",
    "DNA binding" = "dnaBinding",
    "Region" = "region",
    "Coiled coil" = "coiledCoil",
    "Motif" = "motif",
    "Compositional bias" = "compositionalBias",
    "Topological domain" = "topologicalDomain",
    "Transmembrane" = "transmembrane",
    "Intramembrane" = "intramembrane",
    "Helix" = "helix",
    "Beta strand" = "betaStrand",
    "Turn" = "turn"
  )
  ptm_types <- c(
    "Modified residue" = "modifiedResidue",
    "Lipidation" = "lipidation",
    "Glycosylation" = "glycosylation",
    "Cross-link" = "crossLink",
    "Disulfide bond" = "disulfideBond"
  )
  processing_types <- c(
    "Signal peptide" = "signalPeptide",
    "Transit peptide" = "transitPeptide",
    "Propeptide" = "propeptide",
    "Chain" = "chain",
    "Peptide" = "peptide",
    "Initiator methionine" = "initiatorMethionine"
  )

  # -- Feature parsers ----
  loc_start <- function(f) as.integer(f[["location"]][["start"]][["value"]])
  loc_end <- function(f) as.integer(f[["location"]][["end"]][["value"]])

  parse_site <- function(f) {
    A3Site(
      index = A3Position(data = loc_start(f)),
      type = site_types[[f[["type"]]]]
    )
  }

  parse_region <- function(f) {
    A3Region(
      index = A3Range(data = rbind(c(loc_start(f), loc_end(f)))),
      type = region_types[[f[["type"]]]]
    )
  }

  parse_ptm <- function(f) {
    a3_type <- ptm_types[[f[["type"]]]]
    # Disulfide bond encodes a residue pair as a two-position range
    idx <- if (f[["type"]] == "Disulfide bond") {
      A3Range(data = rbind(c(loc_start(f), loc_end(f))))
    } else {
      A3Position(data = loc_start(f))
    }
    A3PTM(index = idx, type = a3_type)
  }

  parse_processing <- function(f) {
    a3_type <- processing_types[[f[["type"]]]]
    # Initiator methionine is a single-residue site; all others are ranges
    idx <- if (f[["type"]] == "Initiator methionine") {
      A3Position(data = loc_start(f))
    } else {
      A3Range(data = rbind(c(loc_start(f), loc_end(f))))
    }
    A3Processing(index = idx, type = a3_type)
  }

  parse_variant <- function(f) {
    alt <- f[["alternativeSequence"]]
    xrefs <- f[["featureCrossReferences"]]
    info <- list(
      from = null_or(alt[["originalSequence"]]),
      to = if (!is.null(alt[["alternativeSequences"]])) {
        paste(
          vapply(alt[["alternativeSequences"]], as.character, character(1L)),
          collapse = "/"
        )
      } else {
        ""
      },
      description = null_or(f[["description"]])
    )
    if (!is.null(f[["featureId"]])) {
      info[["featureId"]] <- f[["featureId"]]
    }
    if (!is.null(xrefs) && length(xrefs) > 0L) {
      dbsnp_ids <- vapply(
        Filter(function(x) identical(x[["database"]], "dbSNP"), xrefs),
        function(x) x[["id"]],
        character(1L)
      )
      if (length(dbsnp_ids) > 0L) {
        info[["dbSnp"]] <- paste(dbsnp_ids, collapse = ",")
      }
    }
    A3Variant(position = A3Position(data = loc_start(f)), info = info)
  }

  # -- Dispatch features to families ----
  features <- if (is.null(dat[["features"]])) list() else dat[["features"]]
  ftype_vec <- if (length(features) > 0L) {
    vapply(features, function(f) f[["type"]], character(1L))
  } else {
    character(0L)
  }

  site_list <- lapply(features[ftype_vec %in% names(site_types)], parse_site)
  region_list <- lapply(
    features[ftype_vec %in% names(region_types)],
    parse_region
  )
  ptm_list <- lapply(features[ftype_vec %in% names(ptm_types)], parse_ptm)
  processing_list <- lapply(
    features[ftype_vec %in% names(processing_types)],
    parse_processing
  )
  variant_list <- lapply(
    features[ftype_vec == "Natural variant"],
    parse_variant
  )

  # Assign unique names within each family: "{a3_type}_{n}" (sequential per type)
  name_by_type <- function(lst) {
    if (length(lst) == 0L) {
      return(lst)
    }
    types <- vapply(lst, function(x) x@type, character(1L))
    names(lst) <- paste0(
      types,
      "_",
      as.integer(ave(seq_along(types), types, FUN = seq_along))
    )
    lst
  }

  site_list <- name_by_type(site_list)
  region_list <- name_by_type(region_list)
  ptm_list <- name_by_type(ptm_list)
  processing_list <- name_by_type(processing_list)

  if (verbosity > 0L) {
    msg("Got:", highlight(paste0(description, " [", uniprot_id, "]")))
  }

  A3(
    sequence = A3Sequence(data = sequence),
    annotations = A3Annotation(
      site = site_list,
      region = region_list,
      ptm = ptm_list,
      processing = processing_list,
      variant = variant_list
    ),
    metadata = A3Metadata(
      uniprot_id = uniprot_id,
      description = description,
      reference = reference,
      organism = organism
    )
  )
}

# ::rtemis.a3::
# 2024- EDG rtemis.org

rtemis.a3_version <- utils::packageVersion("rtemis.a3")

.onLoad <- function(libname, pkgname) {
  S7::methods_register()
}

.onAttach <- function(libname, pkgname) {
  packageStartupMessage(
    paste0(
      ".:",
      pkgname,
      " ",
      rtemis.a3_version,
      " \U1F9EC",
      " ",
      utils::sessionInfo()[[2]]
    )
  )
}

#' \pkg{rtemis.a3}: Amino Acid Annotation format
#'
#' @name rtemis.a3-package
#'
#' @title rtemis.a3: Amino Acid Annotation format
#'
#' @description
#' Amino Acid Annotation format utilities
#'
#' @import cli data.table rtemis.core S7 utils
#' @importFrom stats na.exclude ave
#' @importFrom grDevices col2rgb
"_PACKAGE"

NULL

#' Get the coding sequence of a gene
#'
#' @param gene Character vector: One or more HGNC gene symbols.
#' @param organism Character scalar: Organism short name (Ensembl convention,
#' e.g. `"hsapiens"`).
#' @param biomart Character scalar: BioMart name.
#' @param host Character scalar: Host address.
#' @param verbosity Integer: Verbosity level.
#'
#' @return data.frame with columns "gene", "ensembl_transcript_id" and "sequence".
#'
#' @author EDG
#' @export
#'
#' @examples
#' # Requires internet connection and fetches data from Ensembl using biomaRt.
#' \dontrun{
#'   mapt_seq <- gene2sequence("MAPT")
#' }
gene2sequence <- function(
  gene,
  organism = "hsapiens",
  biomart = "ensembl",
  host = "https://www.ensembl.org",
  verbosity = 1L
) {
  check_dependencies("biomaRt")
  check_character(gene, arg_name = "gene")
  check_scalar_character(organism, arg_name = "organism")
  check_scalar_character(biomart, arg_name = "biomart")
  check_scalar_character(host, arg_name = "host")

  if (verbosity > 0) {
    msg0("Getting sequence for gene ", highlight(gene), "...")
  }

  # Mart ----
  mart <- biomaRt::useMart(
    biomart = biomart,
    dataset = paste(organism, "_gene_ensembl", sep = ""),
    host = host
  )

  # Get transcript ID ----
  # Use gene name as filter and get transcript ID (ensembl_transcript_id)
  transcripts <- biomaRt::getBM(
    attributes = c("ensembl_gene_id", "ensembl_transcript_id"),
    filters = "hgnc_symbol",
    values = gene,
    mart = mart
  )

  msg0(
    "Found ",
    bold(nrow(transcripts)),
    " transcripts for gene ",
    highlight(gene),
    ".",
    verbosity = verbosity
  )

  # Get sequence ----
  # Retrieve sequence(s) using transcript ID
  sequence <- biomaRt::getSequence(
    id = transcripts[["ensembl_transcript_id"]],
    type = "ensembl_transcript_id",
    seqType = "coding",
    mart = mart,
    verbose = verbosity > 1
  )

  if (verbosity > 0) {
    # Count number of sequences returned that are not "Sequence unavailable"
    nretrieved <- sum(sequence[["coding"]] != "Sequence unavailable")
    msg0(
      "Database returned sequences for ",
      bold(nretrieved),
      "/",
      nrow(sequence),
      " transcripts."
    )
  }

  seq <- data.frame(
    gene = gene,
    ensembl_transcript_id = sequence[["ensembl_transcript_id"]],
    sequence = sequence[["coding"]]
  )
  seq
}

# %% pdb_annotations ----
#' Fetch PDB structural annotations for a UniProt accession
#'
#' Queries the PDBe APIs for secondary structure and ligand binding sites from
#' experimental PDB structures, converts residue numbers to UniProt canonical
#' coordinates via SIFTS, and returns named lists of `A3Region` and `A3Site`
#' objects ready for use with `create_A3()`.
#'
#' Structure ranking and SIFTS residue mappings come from the PDBe Graph API
#' `best_structures` endpoint. Secondary structure and binding sites are fetched
#' from the PDBe REST API.
#'
#' When `pdb_id` is `NULL` the top-ranked structure (by PDBe coverage score) is
#' used automatically.
#'
#' @param accession Character scalar: UniProt accession, e.g. `"P10636"`.
#' @param pdb_id Optional character scalar: Four-character PDB ID (e.g. `"2mz7"`). If
#'   `NULL`, the top-ranked structure from PDBe is used.
#' @param pdbe_graph_url Character scalar: PDBe Graph API base URL.
#' @param pdbe_api_url Character scalar: PDBe REST API base URL.
#' @param verbosity Integer scalar: Verbosity level.
#'
#' @return Named list with two elements:
#'   \describe{
#'     \item{`region`}{Named list of `A3Region` objects for secondary structure
#'       (types: `"helix"`, `"betaStrand"`, `"turn"`).}
#'     \item{`site`}{Named list of `A3Site` objects for ligand binding sites
#'       (type: `"bindingSite"`).}
#'   }
#'
#' @author EDG
#' @export
#'
#' @examples
#' # Requires internet connection and fetches data from PDBe.
#' \dontrun{
#'   ann  <- pdb_annotations("P10636")
#'   mapt <- create_A3(
#'     sequence = uniprot_sequence("P10636"),
#'     region   = ann[["region"]],
#'     site     = ann[["site"]]
#'   )
#' }
pdb_annotations <- function(
  accession,
  pdb_id = NULL,
  pdbe_graph_url = "https://www.ebi.ac.uk/pdbe/graph-api",
  pdbe_api_url = "https://www.ebi.ac.uk/pdbe/api",
  verbosity = 1L
) {
  check_dependencies(c("httr", "jsonlite"))
  check_scalar_character(accession)
  check_optional_scalar_character(pdb_id)
  check_scalar_character(pdbe_graph_url)
  check_scalar_character(pdbe_api_url)

  fetch_json <- function(url) {
    resp <- httr::GET(url)
    httr::stop_for_status(resp)
    jsonlite::fromJSON(
      httr::content(resp, as = "text", encoding = "UTF-8"),
      simplifyVector = FALSE
    )
  }

  # -- SIFTS mappings via Graph API `best_structures` ----
  # Returns a flat list of segments, each with:
  #   pdb_id, chain_id, unp_start, unp_end, pdb_start, pdb_end, coverage
  # PDBe ranks them by coverage then resolution, so the first entry is best.
  best_dat <- fetch_json(paste0(
    pdbe_graph_url,
    "/mappings/best_structures/",
    accession
  ))
  all_segs <- best_dat[[accession]]

  if (is.null(all_segs) || length(all_segs) == 0L) {
    cli::cli_abort("No PDB structures found for accession {.val {accession}}.")
  }

  # -- Select structure ----
  if (is.null(pdb_id)) {
    # First entry is PDBe's top-ranked structure
    pdb_id <- tolower(all_segs[[1L]][["pdb_id"]])
    msg(
      "Selected PDB structure:",
      highlight(toupper(pdb_id)),
      "(PDBe top-ranked)",
      verbosity = verbosity
    )
  } else {
    pdb_id <- tolower(pdb_id)
    seg_ids <- vapply(
      all_segs,
      function(s) tolower(s[["pdb_id"]]),
      character(1L)
    )
    if (!pdb_id %in% seg_ids) {
      cli::cli_abort(
        "PDB ID {.val {pdb_id}} not found in best_structures for {.val {accession}}."
      )
    }
  }

  # Collect all segments for the chosen structure (may span multiple chains)
  segments <- Filter(
    function(s) identical(tolower(s[["pdb_id"]]), pdb_id),
    all_segs
  )

  mapped_chains <- unique(vapply(
    segments,
    function(s) s[["chain_id"]],
    character(1L)
  ))
  unp_start_global <- min(vapply(
    segments,
    function(s) s[["unp_start"]],
    integer(1L)
  ))
  unp_end_global <- max(vapply(
    segments,
    function(s) s[["unp_end"]],
    integer(1L)
  ))

  # -- Coordinate converter: PDB author residue number → UniProt ----
  # best_structures uses "start"/"end" for PDB author residue numbers.
  # For each segment: uniprot_pos = pdb_pos - start + unp_start
  convert_pos <- function(pdb_positions, chain) {
    result <- rep(NA_integer_, length(pdb_positions))
    for (seg in segments) {
      if (!identical(seg[["chain_id"]], chain)) {
        next
      }
      ps <- seg[["start"]]
      pe <- seg[["end"]]
      in_seg <- pdb_positions >= ps & pdb_positions <= pe
      result[in_seg] <- pdb_positions[in_seg] - ps + seg[["unp_start"]]
    }
    result
  }

  # -- Secondary structure from PDBe REST API ----
  ss_raw <- fetch_json(paste0(
    pdbe_api_url,
    "/pdb/entry/secondary_structure/",
    pdb_id
  ))
  molecules <- ss_raw[[pdb_id]][["molecules"]]

  region_list <- list()

  for (mol in molecules) {
    for (chain in mol[["chains"]]) {
      chain_id <- chain[["chain_id"]]
      if (!chain_id %in% mapped_chains) {
        next
      }
      ss <- chain[["secondary_structure"]]

      parse_ss_elements <- function(elements, a3_type) {
        lapply(elements, function(el) {
          s <- as.integer(el[["start"]][["author_residue_number"]])
          e <- as.integer(el[["end"]][["author_residue_number"]])
          unp <- convert_pos(c(s, e), chain_id)
          if (
            anyNA(unp) ||
              unp[[1L]] < unp_start_global ||
              unp[[2L]] > unp_end_global ||
              unp[[1L]] >= unp[[2L]]
          ) {
            return(NULL)
          }
          A3Region(
            index = A3Range(data = matrix(unp, nrow = 1L)),
            type = a3_type
          )
        })
      }

      region_list <- c(
        region_list,
        Filter(Negate(is.null), parse_ss_elements(ss[["helices"]], "helix")),
        Filter(
          Negate(is.null),
          parse_ss_elements(ss[["strands"]], "betaStrand")
        ),
        Filter(Negate(is.null), parse_ss_elements(ss[["turns"]], "turn"))
      )
    }
  }

  # -- Binding sites from PDBe REST API ----
  bs_resp <- httr::GET(paste0(
    pdbe_api_url,
    "/pdb/entry/binding_sites/",
    pdb_id
  ))
  site_list <- list()

  # 404 = no binding sites; anything else is an error
  if (httr::status_code(bs_resp) != 404L) {
    httr::stop_for_status(bs_resp)
    binding_sites <- jsonlite::fromJSON(
      httr::content(bs_resp, as = "text", encoding = "UTF-8"),
      simplifyVector = FALSE
    )[[pdb_id]]

    for (bs in binding_sites) {
      residues <- Filter(
        function(r) {
          identical(r[["symmetry_symbol"]], "1_555") &&
            r[["chain_id"]] %in% mapped_chains
        },
        bs[["residues"]]
      )
      if (length(residues) == 0L) {
        next
      }

      chain_id <- residues[[1L]][["chain_id"]]
      unp_positions <- sort(unique(Filter(
        function(p) !is.na(p) & p >= unp_start_global & p <= unp_end_global,
        convert_pos(
          vapply(
            residues,
            function(r) as.integer(r[["author_residue_number"]]),
            integer(1L)
          ),
          chain_id
        )
      )))
      if (length(unp_positions) == 0L) {
        next
      }

      site_list <- c(
        site_list,
        list(
          A3Site(index = A3Position(data = unp_positions), type = "bindingSite")
        )
      )
    }
  }

  # -- Name by type with sequential suffix ----
  name_by_type <- function(lst) {
    if (length(lst) == 0L) {
      return(lst)
    }
    types <- vapply(lst, function(x) x@type, character(1L))
    names(lst) <- paste0(
      types,
      "_",
      as.integer(ave(seq_along(types), types, FUN = seq_along))
    )
    lst
  }

  region_list <- name_by_type(region_list)
  site_list <- name_by_type(site_list)

  msg(
    "Parsed",
    highlight(length(region_list)),
    "secondary structure regions and",
    highlight(length(site_list)),
    "binding sites from",
    highlight(toupper(pdb_id)),
    verbosity = verbosity
  )

  list(region = region_list, site = site_list)
}

# %% rtemis_color_system ----
rtemis_light_teal <- "#00fdfd"
rtemis_teal <- "#00b2b2"
rtemis_light_blue <- "#30cefe"
kaimana_red <- "#ff004c"
kaimana_blue <- "#0067e0"
kaimana_light_blue <- "#479cff"
coastside_orange <- "#ff9f20"
rtemis_orange <- "#ff4f36"
kaimana_green <- "#00ffb3"
kaimana_med_green <- "#00996b"
rtemis_purple <- "#6125f7"
rtemis_magenta <- "#912ac8"
rtemis_light_magenta <- "#b25bd6"
magenta <- "#ff00ff"
lmd_burgundy <- "#a92459"
rtms_gray <- "#808080"


# %% internal colors ----
rt_gray <- rtms_gray
rt_red <- kaimana_red
rt_blue <- kaimana_light_blue
rt_green <- kaimana_med_green
rt_orange <- coastside_orange
rt_teal <- rtemis_teal
rt_purple <- rtemis_purple
rt_magenta <- rtemis_light_magenta
highlight_col <- coastside_orange
col_object <- rt_green


# %% rtemis_colors ----
rtemis_colors <- list(
  red = kaimana_red,
  light_blue = kaimana_light_blue,
  blue = kaimana_blue,
  light_green = kaimana_green,
  green = kaimana_med_green,
  orange = coastside_orange,
  light_teal = rtemis_light_teal,
  teal = rtemis_teal,
  purple = rtemis_purple,
  light_magenta = rtemis_light_magenta,
  magenta = rtemis_magenta
)

# A3 S7 implementation
#  A3
#  ├── sequence:        A3Sequence
#  ├── annotations:     A3Annotation
#  │   ├── site:        list of A3Site
#  │   ├── region:      list of A3Region
#  │   ├── ptm:         list of A3PTM
#  │   ├── processing:  list of A3Processing
#  │   └── variant:     list of A3Variant
#  └── metadata:        A3Metadata
#      ├── uniprot_id:  character(1)
#      ├── description: character(1)
#      ├── reference:   character(1)
#      └── organism:    character(1)
# 2026- EDG rtemis.org

.A3_SCHEMA_URI <- "https://schema.rtemis.org/a3/v1/schema.json"
.A3_VERSION <- "1.0.0"

# %% Level 1: A3Sequence ----
A3Sequence <- new_class(
  "A3Sequence",
  properties = list(
    data = class_character
  ),
  constructor = function(data) {
    new_object(
      S7_object,
      data = toupper(data)
    )
  },
  validator = function(self) {
    if (length(self@data) > 1) {
      cli::cli_abort("Sequence must be a single string.")
    }
    if (nchar(self@data) < 2) {
      cli::cli_abort("Sequence must be at least 2 characters long.")
    }
    if (!grepl("^[A-Z*]+$", self@data)) {
      cli::cli_abort(
        "Sequence must only contain uppercase letters [A-Z] and '*'."
      )
    }
  }
)


# %% to_base.A3Sequence ----
#' Convert A3Sequence to base character string
#'
#' @param x A3Sequence object
#' @return Character string of the sequence
#' @author EDG
#' @keywords internal
#' @noRd
method(to_base, A3Sequence) <- function(x) {
  x@data
}


# %% A3Index ----
# Abstract superclass for A3Position and A3Range, encoding the location of `A3Feature` objects.
A3Index <- new_class(
  "A3Index",
  properties = list(
    data = class_integer
  ),
  abstract = TRUE
)


# %% A3Position ----
# A3Index subclass for individual residue annotations
# Used by `A3Site`, `A3PTM`, and `A3Processing`.
A3Position <- new_class(
  "A3Position",
  parent = A3Index,
  constructor = function(data) {
    new_object(
      S7_object(),
      data = sort(data)
    )
  },
  validator = function(self) {
    if (any(self@data < 1)) {
      cli::cli_abort("Position data must be a positive integer.")
    }
    if (any(duplicated(self@data))) {
      cli::cli_abort("Position data must be unique.")
    }
  }
)


# %% A3Range ----
# A3Index subclass for range annotations
A3Range <- new_class(
  "A3Range",
  parent = A3Index,
  constructor = function(data) {
    # Sort by start position
    idx <- order(data[, 1])
    colnames(data) <- c("start", "end")
    new_object(
      S7_object(),
      data = data[idx, , drop = FALSE]
    )
  },
  validator = function(self) {
    if (ncol(self@data) != 2L) {
      cli::cli_abort("Range must be a matrix with 2 columns.")
    }
    if (any(self@data[, 1] >= self@data[, 2])) {
      cli::cli_abort("Start of range must be less than end of range.")
    }
    if (any(self@data < 1L)) {
      cli::cli_abort("Range values must be positive integers.")
    }
    if (nrow(self@data) > 1L) {
      starts <- self@data[-1L, 1L]
      ends <- self@data[-nrow(self@data), 2L]
      overlap_idx <- which(starts <= ends)
      if (length(overlap_idx) > 0L) {
        i <- overlap_idx[[1L]]
        cli::cli_abort(
          "Range entries must not overlap. \\
          Found: [{self@data[i, 1]}, {self@data[i, 2]}] and \\
          [{self@data[i + 1L, 1]}, {self@data[i + 1L, 2]}]."
        )
      }
    }
  }
)


# %% A3Feature ----
# Abstract base class for annotation feature types.
A3Feature <- new_class(
  "A3Feature",
  properties = list(
    type = class_character
  ),
  abstract = TRUE
)


# %% A3Site ----
A3Site <- new_class(
  "A3Site",
  parent = A3Feature,
  properties = list(
    index = A3Position
  ),
  constructor = function(index, type = "") {
    new_object(
      S7_object(),
      index = index,
      type = type
    )
  }
)


# %% A3Region ----
A3Region <- new_class(
  "A3Region",
  parent = A3Feature,
  properties = list(
    index = A3Range
  ),
  constructor = function(index, type = "") {
    new_object(
      S7_object(),
      index = index,
      type = type
    )
  }
)


# %% A3PTM ----
A3PTM <- new_class(
  "A3PTM",
  parent = A3Feature,
  properties = list(
    index = A3Index
  ),
  constructor = function(index, type = "") {
    new_object(
      S7_object(),
      index = index,
      type = type
    )
  }
)


# %% A3Processing ----
A3Processing <- new_class(
  "A3Processing",
  parent = A3Feature,
  properties = list(
    index = A3Index
  ),
  constructor = function(index, type = "") {
    new_object(
      S7_object(),
      index = index,
      type = type
    )
  }
)


# %% A3Variant ----
A3Variant <- new_class(
  "A3Variant",
  properties = list(
    position = A3Position,
    info = class_list
  ),
  validator = function(self) {
    if (length(self@position@data) != 1) {
      cli::cli_abort("Variant position must be a single integer.")
    }
  }
)


# %% Level 1: A3Annotation ----
A3Annotation <- new_class(
  "A3Annotation",
  properties = list(
    site = class_list,
    region = class_list,
    ptm = class_list,
    processing = class_list,
    variant = class_list
  ),
  validator = function(self) {
    check_names <- function(lst, category) {
      nms <- names(lst)
      if (length(lst) > 0 && (is.null(nms) || any(!nzchar(nms)))) {
        cli::cli_abort(
          "All {category} annotation names must be non-empty strings."
        )
      }
    }
    if (!all(sapply(self@site, S7_inherits, A3Site))) {
      cli::cli_abort("All site annotations must be A3Site objects.")
    }
    check_names(self@site, "site")
    if (!all(sapply(self@region, S7_inherits, A3Region))) {
      cli::cli_abort("All region annotations must be A3Region objects.")
    }
    check_names(self@region, "region")
    if (!all(sapply(self@ptm, S7_inherits, A3PTM))) {
      cli::cli_abort("All PTM annotations must be A3PTM objects.")
    }
    check_names(self@ptm, "ptm")
    if (!all(sapply(self@processing, S7_inherits, A3Processing))) {
      cli::cli_abort("All processing annotations must be A3Processing objects.")
    }
    check_names(self@processing, "processing")
    if (!all(sapply(self@variant, S7_inherits, A3Variant))) {
      cli::cli_abort("All variant annotations must be A3Variant objects.")
    }
  }
)


# %% to_base.A3Annotation ----
#' Convert A3Annotation to base R list format
#'
#' @param x A3Annotation object
#' @return Named list with site, region, ptm, processing, and variant annotations
#' @author EDG
#' @keywords internal
#' @noRd
method(to_base, A3Annotation) <- function(x) {
  list(
    site = lapply(x@site, function(site) {
      list(
        type = site@type,
        index = site@index@data
      )
    }),
    region = lapply(x@region, function(region) {
      list(
        type = region@type,
        index = region@index@data
      )
    }),
    ptm = lapply(x@ptm, function(ptm) {
      list(
        type = ptm@type,
        index = ptm@index@data
      )
    }),
    processing = lapply(x@processing, function(proc) {
      list(
        type = proc@type,
        index = proc@index@data
      )
    }),
    variant = lapply(x@variant, function(v) {
      out <- list(position = v@position@data)
      for (nm in names(v@info)) {
        out[[nm]] <- v@info[[nm]]
      }
      out
    })
  )
}


# %% Metadata ----
Metadata <- new_class(
  "Metadata"
)


# %% Level 1: A3Metadata ----
A3Metadata <- new_class(
  "A3Metadata",
  parent = Metadata,
  properties = list(
    uniprot_id = class_character,
    description = class_character,
    reference = class_character,
    organism = class_character
  ),
  constructor = function(
    uniprot_id = "",
    description = "",
    reference = "",
    organism = ""
  ) {
    new_object(
      Metadata,
      uniprot_id = uniprot_id,
      description = description,
      reference = reference,
      organism = organism
    )
  },
  validator = function(self) {
    if (length(self@uniprot_id) != 1L) {
      cli::cli_abort(
        "{.arg uniprot_id} must be a single string (character(1))."
      )
    }
    if (length(self@description) != 1L) {
      cli::cli_abort(
        "{.arg description} must be a single string (character(1))."
      )
    }
    if (length(self@reference) != 1L) {
      cli::cli_abort("{.arg reference} must be a single string (character(1)).")
    }
    if (length(self@organism) != 1L) {
      cli::cli_abort("{.arg organism} must be a single string (character(1)).")
    }
  }
)


# %% to_base.A3Metadata ----
#' Convert A3Metadata to base R list format
#'
#' @param x A3Metadata object
#' @return Named list with uniprot_id, description, reference, and organism
#' @author EDG
#' @keywords internal
#' @noRd
method(to_base, A3Metadata) <- function(x) {
  list(
    uniprot_id = x@uniprot_id,
    description = x@description,
    reference = x@reference,
    organism = x@organism
  )
}


# %% A3 ----
A3 <- new_class(
  "A3",
  properties = list(
    sequence = A3Sequence,
    annotations = A3Annotation,
    metadata = A3Metadata
  ),
  validator = function(self) {
    seq_length <- nchar(self@sequence@data)

    # Validate site annotations
    for (site in self@annotations@site) {
      if (any(site@index@data > seq_length)) {
        cli::cli_abort(
          "Site annotation positions must be within the sequence length."
        )
      }
    }

    # Validate region annotations
    for (region in self@annotations@region) {
      if (any(region@index@data > seq_length)) {
        cli::cli_abort(
          "Region annotation positions must be within the sequence length."
        )
      }
    }

    # Validate PTM annotations
    for (ptm in self@annotations@ptm) {
      if (any(ptm@index@data > seq_length)) {
        cli::cli_abort(
          "PTM annotation positions must be within the sequence length."
        )
      }
    }

    # Validate processing annotations
    for (proc in self@annotations@processing) {
      if (any(proc@index@data > seq_length)) {
        cli::cli_abort(
          "Processing annotation positions must be within the sequence length."
        )
      }
    }

    # Validate variant annotations
    for (variant in self@annotations@variant) {
      if (variant@position@data > seq_length) {
        cli::cli_abort(
          "Variant annotation position must be within the sequence length."
        )
      }
    }
  }
)


# %% `.DollarNames.A3` ----
method(`.DollarNames`, A3) <- function(x, pattern = "") {
  grep(pattern, prop_names(x), value = TRUE)
}


# %% `$.A3` ----
method(`$`, A3) <- function(x, name) {
  to_base(prop(x, name))
}


# %% `[[.A3` ----
method(`[[`, A3) <- function(x, name) {
  to_base(prop(x, name))
}


# %% to_base.A3 ----
method(to_base, A3) <- function(x) {
  list(
    sequence = to_base(x@sequence),
    annotations = to_base(x@annotations),
    metadata = to_base(x@metadata)
  )
}


# %% repr.A3 ----
method(repr, A3) <- function(x, output_type = NULL, head_n = 10L) {
  out <- repr_S7name("A3", output_type = output_type)

  # Metadata
  ## Description
  if (nchar(x@metadata@description) > 0) {
    out <- paste0(
      out,
      "  Description: ",
      bold(x@metadata@description, output_type = output_type),
      "\n"
    )
  }

  ## Uniprot ID
  if (nchar(x@metadata@uniprot_id) > 0) {
    out <- paste0(
      out,
      "   Uniprot ID: ",
      bold(x@metadata@uniprot_id, output_type = output_type),
      "\n"
    )
  }

  ## Organism
  if (nchar(x@metadata@organism) > 0) {
    out <- paste0(
      out,
      "     Organism: ",
      bold(x@metadata@organism, output_type = output_type),
      "\n"
    )
  }

  ## Reference
  if (nchar(x@metadata@reference) > 0) {
    out <- paste0(
      out,
      "    Reference: ",
      bold(x@metadata@reference, output_type = output_type),
      "\n"
    )
  }

  # Sequence
  seq_length <- nchar(x@sequence@data)
  display_seq <- if (seq_length > head_n) {
    paste0(substr(x@sequence@data, 1L, head_n), "...")
  } else {
    x@sequence@data
  }
  out <- paste0(
    out,
    "     Sequence: ",
    bold(display_seq, output_type = output_type),
    " (length = ",
    seq_length,
    ")\n"
  )

  # Annotations
  # Get annotation info
  site_annotations <- names(x@annotations@site)
  region_annotations <- names(x@annotations@region)
  ptm_annotations <- names(x@annotations@ptm)
  n_processing_annotations <- length(x@annotations@processing)
  n_variant_annotations <- length(x@annotations@variant)

  out <- paste0(out, "  Annotations:\n")

  # Check if no annotations
  if (
    is.null(site_annotations) &&
      is.null(region_annotations) &&
      is.null(ptm_annotations) &&
      n_processing_annotations == 0 &&
      n_variant_annotations == 0
  ) {
    out <- paste0(out, gray("             None\n", output_type = output_type))
  }

  ## Site annotations
  if (length(site_annotations) > 0) {
    out <- paste0(
      out,
      "          ",
      gray("Site:", output_type = output_type),
      " ",
      paste(bold(site_annotations, output_type = output_type), collapse = ", "),
      "\n"
    )
  }

  ## Region annotations
  if (length(region_annotations) > 0) {
    out <- paste0(
      out,
      "        ",
      gray("Region:", output_type = output_type),
      " ",
      paste(
        bold(region_annotations, output_type = output_type),
        collapse = ", "
      ),
      "\n"
    )
  }

  ## PTM annotations
  if (length(ptm_annotations) > 0) {
    out <- paste0(
      out,
      "           ",
      gray("PTM:", output_type = output_type),
      " ",
      paste(bold(ptm_annotations, output_type = output_type), collapse = ", "),
      "\n"
    )
  }

  ## Processing annotations
  if (n_processing_annotations > 0) {
    out <- paste0(
      out,
      " ",
      gray("   Processing:", output_type = output_type),
      " ",
      bold(n_processing_annotations, output_type = output_type),
      ngettext(n_processing_annotations, " annotation\n", " annotations\n")
    )
  }

  ## Variant annotations
  if (n_variant_annotations > 0) {
    out <- paste0(
      out,
      "      ",
      gray("Variants:", output_type = output_type),
      " ",
      bold(n_variant_annotations, output_type = output_type),
      ngettext(n_variant_annotations, " annotation\n", " annotations\n")
    )
  }

  out
}


# %% print.A3 ----
method(print, A3) <- function(x, output_type = NULL, ...) {
  cat(repr(x, output_type = output_type))
  invisible(x)
}


# %% Public API ------------------------------------------------------------------------------------
# The public API uses functions to create A3 objects from base R data structures.
# Minimum set of required functions:
# * `create_A3(sequence, annotations, metadata)`
#   * sequence is character(1); can be character(n) that will be concatenated using `concat()`
#   * site: named list created using `annotation_position()`
#   * region: named list created using `annotation_range()`
#   * ptm: named list created using `annotation_position()` OR `annotation_range()`
#   * processing: named list created using `annotation_position()` OR `annotation_range()`
#   * variant: named list created using `annotation_variant()`
#   * uniprot_id: `character(1)`
#   * description: `character(1)`
#   * reference: `character(1)`
#   * organism: `character(1)`

# %% concat ----
#' Concatenate character vector to single string for sequence input
#'
#' @param x Character vector to concatenate
#'
#' @return Single character string
#'
#' @author EDG
#' @export
#'
#' @examples
#' # Concatenate a character vector of single-letter amino acids into a sequence string
#' concat(c("M", "A", "E", "P", "R"))
concat <- function(x) {
  check_inherits(x, "character")
  if (length(x) > 1) {
    if (any(nchar(x) != 1)) {
      cli::cli_abort(
        "All elements of sequence vector must be single characters."
      )
    }
    paste(x, collapse = "")
  } else {
    x
  }
}


# %% annotation_position ----
#' Create position-based annotations for A3
#'
#' Creates an annotation spec (named list) with an `A3Position` index and optional type,
#' for use with `create_A3()`.
#'
#' @param x Integer vector: Positions of the annotation (1-based indexing).
#' @param type Optional character scalar: Annotation type. `NULL` is treated as
#' the empty string in the canonical A3 representation.
#'
#' @return Named list with `index` (A3Position) and `type` (character)
#'
#' @author EDG
#' @export
#'
#' @examples
#' # Create a site annotation for positions 5 and 17, of type "active site"
#' annotation_position(c(5L, 17L), type = "active site")
annotation_position <- function(x, type = NULL) {
  check_optional_scalar_character(type)
  if (is.null(type)) type <- ""
  list(
    index = A3Position(data = clean_int(x)),
    type = type
  )
}

# %% annotation_range ----
#' Create range-based annotations for A3
#'
#' Creates an annotation spec (named list) with an `A3Range` index and optional type,
#' for use with `create_A3()`.
#'
#' @param x Integer matrix with 2 columns corresponding to start and end positions of the
#' annotation (1-based indexing).
#' @param type Optional character scalar: Annotation type. `NULL` is treated as
#' the empty string in the canonical A3 representation.
#'
#' @return Named list with `index` (A3Range) and `type` (character)
#'
#' @author EDG
#' @export
#'
#' @examples
#' # Create a region annotation for ranges 3-10 and 15-22, of type "repeat"
#' annotation_range(rbind(c(3L, 10L), c(15L, 22L)), type = "repeat")
annotation_range <- function(x, type = NULL) {
  check_optional_scalar_character(type)
  if (is.null(type)) type <- ""
  list(
    index = A3Range(data = clean_int(x)),
    type = type
  )
}


# %% annotation_variant ----
#' Create variant annotations for A3
#'
#' Creates an `A3Variant` object for variant annotations with position and info.
#'
#' @param x Integer scalar: Position of the variant (1-based indexing).
#' @param info Named list: Additional information about the variant (e.g. reference and alternate
#' amino acids, variant type, etc.)
#'
#' @return A3Variant object
#'
#' @author EDG
#' @export
#'
#' @examples
#' # Create a variant annotation for position 10 with info about the variant
#' annotation_variant(10L, info = list(ref = "A", alt = "T", type = "missense"))
annotation_variant <- function(x, info = list()) {
  check_inherits(info, "list")
  A3Variant(
    position = A3Position(clean_int(x)),
    info = info
  )
}


# %% create_A3 ----
#' Create an A3 object from sequence, annotations, and metadata
#'
#' @param sequence Character scalar: Amino acid sequence string.
#' @param site Named list of site annotations
#' @param region Named list of region annotations
#' @param ptm Named list of PTM annotations
#' @param processing Named list of processing annotations
#' @param variant Named list of variant annotations
#' @param uniprot_id Optional character scalar: UniProt accession.
#' @param description Optional character scalar: Protein description.
#' @param reference Optional character scalar: Citation or URL.
#' @param organism Optional character scalar: Species name.
#'
#' @return A3 object
#'
#' @author EDG
#' @export
#'
#' @examples
#' # Minimal: sequence only
#' a <- create_A3("MAEPRQEFEVMEDHAGTYGLGDRK")
#'
#' # With site, region, and PTM annotations
#' a <- create_A3(
#'   "MAEPRQEFEVMEDHAGTYGLGDRK",
#'   site = list(
#'     Active_site = annotation_position(c(5L, 17L), type = "active site")
#'   ),
#'   region = list(
#'     Domain = annotation_range(rbind(c(3L, 10L), c(15L, 22L)),
#'       type = "repeat"
#'     )
#'   ),
#'   ptm = list(
#'     Phosphorylation = annotation_position(c(2L, 18L), type = "phosphoserine")
#'   ),
#'   uniprot_id = "P10636",
#'   organism = "Homo sapiens"
#' )
create_A3 <- function(
  sequence,
  site = list(),
  region = list(),
  ptm = list(),
  processing = list(),
  variant = list(),
  uniprot_id = NULL,
  description = NULL,
  reference = NULL,
  organism = NULL
) {
  check_scalar_character(sequence, arg_name = "sequence")
  check_inherits(site, "list")
  check_inherits(region, "list")
  check_inherits(ptm, "list")
  check_inherits(processing, "list")
  check_inherits(variant, "list")
  check_optional_scalar_character(uniprot_id)
  check_optional_scalar_character(description)
  check_optional_scalar_character(reference)
  check_optional_scalar_character(organism)
  if (is.null(uniprot_id))  uniprot_id  <- ""
  if (is.null(description)) description <- ""
  if (is.null(reference))   reference   <- ""
  if (is.null(organism))    organism    <- ""
  site <- lapply(site, function(a) {
    if (S7_inherits(a, A3Site)) {
      a
    } else {
      A3Site(index = a[["index"]], type = a[["type"]])
    }
  })
  region <- lapply(region, function(a) {
    if (S7_inherits(a, A3Region)) {
      a
    } else {
      A3Region(index = a[["index"]], type = a[["type"]])
    }
  })
  ptm <- lapply(ptm, function(a) {
    if (S7_inherits(a, A3PTM)) {
      a
    } else {
      A3PTM(index = a[["index"]], type = a[["type"]])
    }
  })
  processing <- lapply(processing, function(a) {
    if (S7_inherits(a, A3Processing)) {
      a
    } else {
      A3Processing(index = a[["index"]], type = a[["type"]])
    }
  })
  A3(
    sequence = A3Sequence(concat(sequence)),
    annotations = A3Annotation(
      site = site,
      region = region,
      ptm = ptm,
      processing = processing,
      variant = variant
    ),
    metadata = A3Metadata(
      uniprot_id = uniprot_id,
      description = description,
      reference = reference,
      organism = organism
    )
  )
}

# %% to_json.A3 ----
#' Convert an S7 object to a JSON string
#'
#' @param x S7 object
#' @param pretty Logical: if TRUE, pretty-print JSON output.
#' @param ... Additional arguments (currently unused)
#'
#' @return JSON string
#'
#' @author EDG
#' @keywords internal
#' @noRd
method(to_json, A3) <- function(x, pretty = TRUE, ...) {
  check_dependencies("jsonlite")

  feature_to_list <- function(feature) {
    idx <- feature@index
    if (S7_inherits(idx, A3Range)) {
      mat <- idx@data
      index_data <- lapply(
        seq_len(nrow(mat)),
        function(i) as.integer(mat[i, ])
      )
    } else {
      index_data <- as.integer(idx@data)
    }
    list(
      index = index_data,
      type = jsonlite::unbox(feature@type)
    )
  }

  variant_to_list <- function(v) {
    out <- list(position = jsonlite::unbox(as.integer(v@position@data)))
    for (nm in names(v@info)) {
      val <- v@info[[nm]]
      out[[nm]] <- if (length(val) == 1) jsonlite::unbox(val) else val
    }
    out
  }

  force_named <- function(x) {
    if (length(x) == 0) structure(list(), names = character(0)) else x
  }

  lst <- list(
    `$schema` = jsonlite::unbox(.A3_SCHEMA_URI),
    a3_version = jsonlite::unbox(.A3_VERSION),
    sequence = jsonlite::unbox(x@sequence@data),
    annotations = list(
      site = force_named(lapply(x@annotations@site, feature_to_list)),
      region = force_named(lapply(x@annotations@region, feature_to_list)),
      ptm = force_named(lapply(x@annotations@ptm, feature_to_list)),
      processing = force_named(lapply(
        x@annotations@processing,
        feature_to_list
      )),
      variant = lapply(x@annotations@variant, variant_to_list)
    ),
    metadata = list(
      uniprot_id = jsonlite::unbox(x@metadata@uniprot_id),
      description = jsonlite::unbox(x@metadata@description),
      reference = jsonlite::unbox(x@metadata@reference),
      organism = jsonlite::unbox(x@metadata@organism)
    )
  )
  jsonlite::toJSON(lst, pretty = pretty)
}


# %% A3from_json ----
#' Create an A3 object from a JSON string or parsed list
#'
#' Accepts canonical A3 JSON (with `index`/`type` fields per annotation) and
#' legacy bare-array format (positions as flat arrays, ranges as arrays of arrays).
#'
#' @param x Character scalar (JSON string) or named list (pre-parsed JSON).
#' @param ... Additional arguments (currently unused)
#'
#' @return A3 object
#'
#' @author EDG
#' @keywords internal
#' @noRd
A3from_json <- function(x, ...) {
  check_dependencies("jsonlite")
  if (is.character(x)) {
    x <- jsonlite::fromJSON(
      x,
      simplifyVector = TRUE,
      simplifyDataFrame = FALSE,
      simplifyMatrix = FALSE
    )
  }

  # Validate required envelope fields
  schema_field <- x[["$schema"]]
  if (is.null(schema_field)) {
    cli::cli_abort("JSON input missing required field {.field $schema}.")
  }
  if (schema_field != .A3_SCHEMA_URI) {
    cli::cli_abort(
      "Field {.field $schema} must be {.val {.A3_SCHEMA_URI}}, got {.val {schema_field}}."
    )
  }
  version_field <- x[["a3_version"]]
  if (is.null(version_field)) {
    cli::cli_abort("JSON input missing required field {.field a3_version}.")
  }
  if (version_field != .A3_VERSION) {
    cli::cli_abort(
      "Field {.field a3_version} must be {.val {.A3_VERSION}}, got {.val {version_field}}."
    )
  }
  if (is.null(x[["sequence"]])) {
    cli::cli_abort("JSON input missing required field {.field sequence}.")
  }
  if (is.null(x[["annotations"]])) {
    cli::cli_abort("JSON input missing required field {.field annotations}.")
  }
  if (is.null(x[["metadata"]])) {
    cli::cli_abort("JSON input missing required field {.field metadata}.")
  }

  sequence <- x[["sequence"]]
  annotations <- x[["annotations"]]

  parse_position_index <- function(idx_data, path) {
    idx <- as.integer(idx_data)
    dup <- unique(idx[duplicated(idx)])
    if (length(dup) > 0L) {
      cli::cli_abort(
        "Field {.field {path}} has duplicate positions: {.val {dup}}. Remove duplicates."
      )
    }

    tryCatch(
      A3Position(data = idx),
      error = function(err) {
        cli::cli_abort(
          "Invalid position index in {.field {path}}: {conditionMessage(err)}"
        )
      }
    )
  }

  parse_range_index <- function(idx_data, path) {
    mat <- do.call(rbind, lapply(idx_data, as.integer))

    tryCatch(
      A3Range(data = mat),
      error = function(err) {
        cli::cli_abort(
          "Invalid range index in {.field {path}}: {conditionMessage(err)}"
        )
      }
    )
  }

  # Parse a single annotation entry (canonical {index, type} form only)
  parse_feature <- function(
    entry,
    feature_class,
    annotation_group,
    annotation_name
  ) {
    if (!is.list(entry) || is.null(entry[["index"]])) {
      cli::cli_abort(
        "Each annotation entry must be an object with 'index' and 'type' fields."
      )
    }
    idx_data <- entry[["index"]]
    type <- if (is.null(entry[["type"]])) "" else entry[["type"]]
    index_path <- paste0(
      "annotations.",
      annotation_group,
      ".",
      annotation_name,
      ".index"
    )

    # List of 2-element vectors → range pairs; plain vector → positions
    if (is.list(idx_data)) {
      index <- parse_range_index(idx_data, index_path)
    } else {
      index <- parse_position_index(idx_data, index_path)
    }

    feature_class(index = index, type = type)
  }

  site <- if (!is.null(annotations[["site"]])) {
    Map(
      parse_feature,
      entry = annotations[["site"]],
      annotation_name = names(annotations[["site"]]),
      MoreArgs = list(
        feature_class = A3Site,
        annotation_group = "site"
      )
    )
  } else {
    list()
  }

  region <- if (!is.null(annotations[["region"]])) {
    Map(
      parse_feature,
      entry = annotations[["region"]],
      annotation_name = names(annotations[["region"]]),
      MoreArgs = list(
        feature_class = A3Region,
        annotation_group = "region"
      )
    )
  } else {
    list()
  }

  ptm <- if (!is.null(annotations[["ptm"]])) {
    Map(
      parse_feature,
      entry = annotations[["ptm"]],
      annotation_name = names(annotations[["ptm"]]),
      MoreArgs = list(
        feature_class = A3PTM,
        annotation_group = "ptm"
      )
    )
  } else {
    list()
  }

  processing <- if (!is.null(annotations[["processing"]])) {
    Map(
      parse_feature,
      entry = annotations[["processing"]],
      annotation_name = names(annotations[["processing"]]),
      MoreArgs = list(
        feature_class = A3Processing,
        annotation_group = "processing"
      )
    )
  } else {
    list()
  }

  # Parse variants
  variants_data <- annotations[["variant"]]
  variant <- if (is.null(variants_data) || length(variants_data) == 0) {
    list()
  } else if (is.data.frame(variants_data)) {
    lapply(seq_len(nrow(variants_data)), function(i) {
      row <- as.list(variants_data[i, ])
      pos <- as.integer(row[["position"]])
      info <- row[setdiff(names(row), "position")]
      A3Variant(position = A3Position(data = pos), info = info)
    })
  } else {
    lapply(variants_data, function(v) {
      v <- as.list(v)
      pos <- as.integer(v[["position"]])
      info <- v[setdiff(names(v), "position")]
      A3Variant(position = A3Position(data = pos), info = info)
    })
  }

  # Parse metadata (all fields default to "")
  meta <- x[["metadata"]]
  null_to_empty <- function(val) if (is.null(val)) "" else val

  A3(
    sequence = A3Sequence(data = sequence),
    annotations = A3Annotation(
      site = site,
      region = region,
      ptm = ptm,
      processing = processing,
      variant = variant
    ),
    metadata = A3Metadata(
      uniprot_id = null_to_empty(meta[["uniprot_id"]]),
      description = null_to_empty(meta[["description"]]),
      reference = null_to_empty(meta[["reference"]]),
      organism = null_to_empty(meta[["organism"]])
    )
  )
}


#' Write `A3` object to JSON file
#'
#' @param x `A3` object.
#' @param filepath Character: Path to save JSON file.
#' @param overwrite Logical: If TRUE, overwrite existing file.
#'
#' @return Invisible `x`. Writes JSON file as side effect.
#'
#' @author EDG
#' @export
#'
#' @examples
#' a3 <- create_A3("MAEPRQEFEVMEDHAGTYGLGDRK", uniprot_id = "P10636")
#' tmp <- tempfile(fileext = ".json")
#' write_A3json(a3, tmp)
#' a3_read <- read_A3json(tmp)
#' unlink(tmp)
write_A3json <- function(x, filepath, overwrite = FALSE) {
  check_is_S7(x, A3)
  check_scalar_character(filepath, arg_name = "filepath")
  check_scalar_logical(overwrite, arg_name = "overwrite")
  filepath <- normalizePath(filepath, mustWork = FALSE)
  if (file.exists(filepath) && !overwrite) {
    cli::cli_abort(
      "File {.file {filepath}} exists. Set {.arg overwrite = TRUE} to overwrite."
    )
  }
  writeLines(to_json(x), filepath)
  invisible(x)
}

#' Read `A3` object from JSON file
#'
#' @param filepath Character: Path to JSON file.
#' @param verbosity Integer: if greater than 0, print messages.
#'
#' @return `A3` object.
#'
#' @author EDG
#' @export
#'
#' @examples
#' a3 <- create_A3("MAEPRQEFEVMEDHAGTYGLGDRK", uniprot_id = "P10636")
#' tmp <- tempfile(fileext = ".json")
#' write_A3json(a3, tmp)
#' a3_read <- read_A3json(tmp, verbosity = 0L)
#' unlink(tmp)
read_A3json <- function(filepath, verbosity = 1L) {
  check_scalar_character(filepath, arg_name = "filepath")
  filepath <- normalizePath(filepath)
  if (!file.exists(filepath)) {
    cli::cli_abort("File {.file {filepath}} does not exist.")
  }
  json_str <- paste(readLines(filepath, warn = FALSE), collapse = "\n")
  obj <- A3from_json(json_str)
  if (verbosity > 0L) {
    msg(
      "Read ",
      basename(filepath),
      ": ",
      fmt("\u2714 ", col = rtemis_colors[["green"]]),
      "valid A3 ",
      .A3_VERSION,
      sep = ""
    )
  }
  obj
}

# %% A3Sequence ----
test_that("A3Sequence succeeds with valid data", {
  x <- A3Sequence(data = "MKTAYIAKQRQISFVKSHFSRQDILDLWIYHTQGYFPDWQNYTPG")
  expect_s7_class(x, A3Sequence)
})

test_that("A3Sequence fails with invalid data", {
  expect_error(
    A3Sequence(data = c("SRQ", "MKTA")),
    "Sequence must be a single string"
  )
  expect_error(
    A3Sequence(data = "M"),
    "Sequence must be at least 2 characters long"
  )
  expect_error(
    A3Sequence(data = "MK123"),
    "Sequence must only contain uppercase letters"
  )
})


# %% A3Index ----
test_that("A3Index is abstract and cannot be instantiated", {
  expect_error(A3Index(data = c(3L, 5L, 7L)))
})


# %% A3Position ----
test_that("A3Position succeeds with valid data", {
  x <- A3Position(data = c(3L, 5L, 7L))
  expect_s7_class(x, A3Position)
})

test_that("A3Position fails with invalid data", {
  expect_error(
    A3Position(data = c(0L, 5L, 7L)),
    "Position data must be a positive integer."
  )
  expect_error(
    A3Position(data = c(-1L, 5L, 7L)),
    "Position data must be a positive integer."
  )
  expect_error(
    A3Position(data = c(1.5, 5L, 7L)),
    "object properties are invalid"
  )
})


# %% A3Range ----
test_that("A3Range succeeds with valid data", {
  # Single range
  x <- A3Range(data = rbind(c(1L, 5L)))
  expect_s7_class(x, A3Range)
  # Two non-overlapping ranges: [1,5] and [10,15]
  x <- A3Range(data = rbind(c(1L, 5L), c(10L, 15L)))
  expect_s7_class(x, A3Range)
  # Adjacent ranges are permitted: [1,5] and [6,10]
  x <- A3Range(data = rbind(c(1L, 5L), c(6L, 10L)))
  expect_s7_class(x, A3Range)
})

test_that("A3Range fails with invalid data", {
  expect_error(
    A3Range(data = rbind(c(0L, 10L), c(5L, 15L))),
    "Range values must be positive integers."
  )
  expect_error(
    A3Range(data = rbind(c(1.5, 10L), c(5L, 15L))),
    "object properties are invalid"
  )
  expect_error(
    A3Range(data = rbind(c(5L, 1L), c(10L, 15L))),
    "Start of range must be less than end of range."
  )
  # Overlapping ranges: [1,5] and [3,8]
  expect_error(
    A3Range(data = rbind(c(1L, 5L), c(3L, 8L))),
    "must not overlap"
  )
})

# %% A3Feature ----
test_that("A3Feature is abstract and cannot be instantiated", {
  expect_error(A3Feature(type = "phosphorylation"))
})


# %% A3Site ----
test_that("A3Site succeeds with valid index and optional type", {
  x <- A3Site(index = A3Position(data = c(3L, 5L, 7L)))
  expect_s7_class(x, A3Site)
  x <- A3Site(
    index = A3Position(data = c(3L, 5L, 7L)),
    type = "phosphorylation"
  )
  expect_s7_class(x, A3Site)
})

test_that("A3Site fails with invalid index or type", {
  expect_error(
    A3Site(index = A3Position(data = c(0L, 5L, 7L))),
    "Position data must be a positive integer."
  )
  expect_error(
    A3Site(index = A3Position(data = c(1.5, 5L, 7L))),
    "object properties are invalid"
  )
  expect_error(
    A3Site(index = A3Position(data = c(1.5, 5L, 7L)), type = 99L),
    "object properties are invalid"
  )
})


# %% A3Region ----
test_that("A3Region succeeds with valid index and optional type", {
  x <- A3Region(index = A3Range(data = rbind(c(1L, 5L), c(10L, 12L))))
  expect_s7_class(x, A3Region)
  x <- A3Region(
    index = A3Range(data = rbind(c(1L, 5L), c(10L, 12L))),
    type = "Phosphodegron"
  )
  expect_s7_class(x, A3Region)
})

test_that("A3Region fails with invalid index or type", {
  expect_error(
    A3Region(index = A3Range(data = rbind(c(0L, 5L), c(10L, 12L)))),
    "Range values must be positive integers"
  )
  expect_error(
    A3Region(index = A3Range(data = rbind(c(1.5, 5L), c(10L, 12L)))),
    "object properties are invalid"
  )
  expect_error(
    A3Region(index = A3Range(data = rbind(c(5L, 1L), c(10L, 12L)))),
    "Start of range must be less than end of range"
  )
  expect_error(
    A3Region(
      index = A3Range(data = rbind(c(1L, 5L), c(10L, 12L))),
      type = 99L
    ),
    "object properties are invalid"
  )
})


# %% A3PTM ----
test_that("A3PTM succeeds with valid index and optional type", {
  x <- A3PTM(index = A3Position(data = c(3L, 5L, 7L)))
  expect_s7_class(x, A3PTM)
  x <- A3PTM(
    index = A3Range(data = rbind(c(3L, 7L), c(10L, 12L))),
    type = "phosphorylation"
  )
  expect_s7_class(x, A3PTM)
})

test_that("A3PTM fails with invalid index or type", {
  expect_error(
    A3PTM(index = A3Position(data = c(0L, 5L, 7L))),
    "Position data must be a positive integer."
  )
  expect_error(
    A3PTM(index = A3Position(data = c(1.5, 5L, 7L))),
    "object properties are invalid"
  )
  expect_error(
    A3PTM(index = A3Position(data = c(1.5, 5L, 7L)), type = 99L),
    "object properties are invalid"
  )
})

# %% A3Processing ----
test_that("A3Processing succeeds with valid index and optional type", {
  x <- A3Processing(index = A3Position(data = c(3L, 5L, 7L)))
  expect_s7_class(x, A3Processing)
  x <- A3Processing(
    index = A3Range(data = rbind(c(3L, 7L), c(10L, 12L))),
    type = "signal peptide"
  )
  expect_s7_class(x, A3Processing)
})

test_that("A3Processing fails with invalid index or type", {
  expect_error(
    A3Processing(index = A3Position(data = c(0L, 5L, 7L))),
    "Position data must be a positive integer."
  )
  expect_error(
    A3Processing(index = A3Position(data = c(1.5, 5L, 7L))),
    "object properties are invalid"
  )
  expect_error(
    A3Processing(index = A3Position(data = c(1.5, 5L, 7L)), type = 99L),
    "object properties are invalid"
  )
})


# %% A3Variant ----
test_that("A3Variant succeeds with valid position and info", {
  x <- A3Variant(
    position = A3Position(data = c(3L)),
    info = list(field_a = "alpha", field_b = 123L)
  )
  expect_s7_class(x, A3Variant)
})

test_that("A3Variant fails with invalid position or info", {
  expect_error(
    A3Variant(
      position = A3Position(data = c(1L, 3L)),
      info = list(field_a = "alpha")
    ),
    "Variant position must be a single integer."
  )
})

# %% A3Annotation ----
test_that("A3Annotation succeeds with valid annotations", {
  x <- A3Annotation(
    site = list(activeSite = A3Site(index = A3Position(data = c(3L, 5L))))
  )
  expect_s7_class(x, A3Annotation)
  x <- A3Annotation(
    site = list(activeSite = A3Site(index = A3Position(data = c(3L, 5L)))),
    region = list(
      KXGS = A3Region(
        index = A3Range(data = rbind(c(1L, 10L)))
      )
    )
  )
  expect_s7_class(x, A3Annotation)
  x <- A3Annotation(
    site = list(activeSite = A3Site(index = A3Position(data = c(3L, 5L)))),
    region = list(
      KXGS = A3Region(
        index = A3Range(data = rbind(c(1L, 10L)))
      )
    ),
    ptm = list(Phosphorylation = A3PTM(index = A3Position(data = c(7L))))
  )
  expect_s7_class(x, A3Annotation)
  x <- A3Annotation(
    site = list(activeSite = A3Site(index = A3Position(data = c(3L, 5L)))),
    region = list(
      KXGS = A3Region(
        index = A3Range(data = rbind(c(1L, 10L)))
      )
    ),
    ptm = list(Phosphorylation = A3PTM(index = A3Position(data = c(7L)))),
    processing = list(
      `Signal peptide` = A3Processing(
        index = A3Range(data = rbind(c(20L, 30L)))
      )
    )
  )
  expect_s7_class(x, A3Annotation)
  x <- A3Annotation(
    site = list(activeSite = A3Site(index = A3Position(data = c(3L, 5L)))),
    region = list(
      KXGS = A3Region(
        index = A3Range(data = rbind(c(1L, 10L)))
      )
    ),
    ptm = list(Phosphorylation = A3PTM(index = A3Position(data = c(7L)))),
    processing = list(
      `Signal peptide` = A3Processing(
        index = A3Range(data = rbind(c(20L, 30L)))
      )
    ),
    variant = list(A3Variant(
      position = A3Position(data = c(15L)),
      info = list(mutation = "R15H")
    ))
  )
  expect_s7_class(x, A3Annotation)
})

test_that("A3Annotation fails with invalid data", {
  expect_error(
    A3Annotation(
      site = list("not_a_site"),
      region = list(),
      ptm = list(),
      processing = list(),
      variant = list()
    ),
    "All site annotations must be A3Site objects."
  )
  expect_error(
    A3Annotation(
      site = list(),
      region = list("not_a_region"),
      ptm = list(),
      processing = list(),
      variant = list()
    ),
    "All region annotations must be A3Region objects."
  )
  expect_error(
    A3Annotation(
      site = list(),
      region = list(),
      ptm = list("not_a_ptm"),
      processing = list(),
      variant = list()
    ),
    "All PTM annotations must be A3PTM objects."
  )
  expect_error(
    A3Annotation(
      site = list(),
      region = list(),
      ptm = list(),
      processing = list("not_processing"),
      variant = list()
    ),
    "All processing annotations must be A3Processing objects."
  )
  expect_error(
    A3Annotation(
      site = list(),
      region = list(),
      ptm = list(),
      processing = list(),
      variant = list("not_a_variant")
    ),
    "All variant annotations must be A3Variant objects."
  )
  expect_error(
    A3Annotation(
      site = list(A3Site(index = A3Position(data = c(3L, 5L))))
    ),
    "All site annotation names must be non-empty strings."
  )
  expect_error(
    A3Annotation(
      site = setNames(
        list(A3Site(index = A3Position(data = c(3L, 5L)))),
        ""
      )
    ),
    "All site annotation names must be non-empty strings."
  )
})


# %% Metadata ----
test_that("Metadata can be instantiated", {
  x <- Metadata()
  expect_s7_class(x, Metadata)
})


# %% A3Metadata ----
test_that("A3Metadata can be instantiated", {
  x <- A3Metadata()
  expect_s7_class(x, A3Metadata)
  expect_identical(x@uniprot_id, "")
  expect_identical(x@description, "")
  expect_identical(x@reference, "")
  expect_identical(x@organism, "")
})

test_that("A3Metadata fails with invalid data", {
  expect_error(
    A3Metadata(uniprot_id = 123L),
    "object properties are invalid"
  )
  expect_error(
    A3Metadata(description = 123L),
    "object properties are invalid"
  )
  expect_error(
    A3Metadata(reference = 123L),
    "object properties are invalid"
  )
  expect_error(
    A3Metadata(organism = 123L),
    "object properties are invalid"
  )
  expect_error(
    A3Metadata(uniprot_id = c("P10636", "Q9Y3Q8")),
    "uniprot_id.*character\\(1\\)"
  )
  expect_error(
    A3Metadata(organism = c("Homo sapiens", "Mus musculus")),
    "organism.*character\\(1\\)"
  )
})


# %% A3 ----
test_that("A3 can be instantiated with valid sequence and annotations", {
  x <- A3(
    sequence = A3Sequence(data = "MKTAYIAKQRQISFVK"),
    annotations = A3Annotation(
      site = list(activeSite = A3Site(index = A3Position(data = c(3L, 5L)))),
      region = list(
        KXGS = A3Region(
          index = A3Range(data = rbind(c(1L, 10L)))
        )
      ),
      ptm = list(Phosphorylation = A3PTM(index = A3Position(data = c(7L)))),
      processing = list(
        `Signal peptide` = A3Processing(
          index = A3Range(data = rbind(c(8L, 12L)))
        )
      ),
      variant = list(A3Variant(
        position = A3Position(data = c(15L)),
        info = list(mutation = "R15H")
      ))
    )
  )
  expect_s7_class(x, A3)
})

test_that("A3 fails with invalid sequence or annotations", {
  expect_error(
    A3(
      sequence = "not_a_sequence",
      annotations = A3Annotation()
    ),
    "object properties are invalid"
  )
  expect_error(
    A3(
      sequence = A3Sequence(data = "MKTAYIAKQRQISFVK"),
      annotations = "not_annotations"
    ),
    "object properties are invalid"
  )
})


# %% create_A3 ----
test_that("create_A3 succeeds with valid inputs", {
  x <- create_A3(
    sequence = "MKTAYIAKQRQISFVK",
    site = list(
      `N-terminal repeat` = annotation_position(
        c(3, 5)
      )
    ),
    region = list(
      Phosphodegron = annotation_range(
        rbind(c(1, 10)),
        type = "functional region"
      )
    ),
    ptm = list(
      Phosphorylation = annotation_position(
        c(7)
      )
    ),
    processing = list(
      `Signal peptide` = annotation_range(
        rbind(c(8, 12))
      )
    ),
    variant = list(
      zdorg = annotation_variant(
        15,
        info = list(mutation = "R15H")
      )
    ),
    uniprot_id = "P12345",
    description = "Example protein",
    reference = "PMID:12345678",
    organism = "Homo sapiens"
  )
  expect_s7_class(x, A3)
})


# %% to_json / A3from_json ----
test_that("to_json produces valid JSON with canonical structure", {
  x <- create_A3(
    sequence = "MKTAYIAKQRQISFVK",
    site = list(
      `Active site` = annotation_position(c(3, 5), type = "activeSite")
    ),
    region = list(
      KXGS = annotation_range(rbind(c(1L, 10L)))
    ),
    ptm = list(
      Phosphorylation = annotation_position(c(7))
    ),
    processing = list(
      `Signal peptide` = annotation_range(rbind(c(8L, 12L)))
    ),
    variant = list(
      annotation_variant(15, info = list(from = "R", to = "H"))
    ),
    uniprot_id = "P12345",
    description = "Example protein",
    organism = "Homo sapiens"
  )
  json <- to_json(x)
  expect_type(json, "character")
  parsed <- jsonlite::fromJSON(json, simplifyVector = FALSE)
  expect_equal(parsed$sequence, "MKTAYIAKQRQISFVK")
  expect_equal(parsed$annotations$site$`Active site`$type, "activeSite")
  expect_equal(parsed$annotations$region$KXGS$index, list(list(1L, 10L)))
  expect_equal(parsed$metadata$uniprot_id, "P12345")
  expect_equal(parsed$metadata$organism, "Homo sapiens")
  expect_equal(parsed$metadata$reference, "")
})

test_that("A3from_json round-trips to_json with zero loss", {
  original <- create_A3(
    sequence = "MKTAYIAKQRQISFVK",
    site = list(
      `Active site` = annotation_position(c(3, 5), type = "activeSite")
    ),
    region = list(
      KXGS = annotation_range(rbind(c(1L, 10L)))
    ),
    ptm = list(
      Phosphorylation = annotation_position(c(7))
    ),
    processing = list(
      `Signal peptide` = annotation_range(rbind(c(8L, 12L)))
    ),
    variant = list(
      annotation_variant(15, info = list(from = "R", to = "H"))
    ),
    uniprot_id = "P12345",
    description = "Example protein",
    reference = "PMID:12345678",
    organism = "Homo sapiens"
  )

  restored <- A3from_json(to_json(original))

  # Sequence
  expect_identical(restored@sequence@data, original@sequence@data)

  # Site annotations
  expect_identical(
    names(restored@annotations@site),
    names(original@annotations@site)
  )
  expect_identical(
    restored@annotations@site[[1]]@index@data,
    original@annotations@site[[1]]@index@data
  )
  expect_identical(
    restored@annotations@site[[1]]@type,
    original@annotations@site[[1]]@type
  )

  # Region annotations
  expect_identical(
    names(restored@annotations@region),
    names(original@annotations@region)
  )
  expect_identical(
    restored@annotations@region[[1]]@index@data,
    original@annotations@region[[1]]@index@data
  )

  # PTM annotations
  expect_identical(
    restored@annotations@ptm[[1]]@index@data,
    original@annotations@ptm[[1]]@index@data
  )

  # Processing annotations
  expect_identical(
    restored@annotations@processing[[1]]@index@data,
    original@annotations@processing[[1]]@index@data
  )

  # Variant annotations
  expect_identical(
    restored@annotations@variant[[1]]@position@data,
    original@annotations@variant[[1]]@position@data
  )
  expect_identical(
    restored@annotations@variant[[1]]@info$from,
    original@annotations@variant[[1]]@info$from
  )

  # Metadata
  expect_identical(restored@metadata@uniprot_id, original@metadata@uniprot_id)
  expect_identical(restored@metadata@description, original@metadata@description)
  expect_identical(restored@metadata@reference, original@metadata@reference)
  expect_identical(restored@metadata@organism, original@metadata@organism)
})

test_that("A3from_json rejects legacy bare-array format", {
  legacy_json <- '{
    "$schema": "https://schema.rtemis.org/a3/v1/schema.json",
    "a3_version": "1.0.0",
    "sequence": "MKTAYIAKQRQISFVK",
    "annotations": {
      "site": {
        "Active site": [3, 5]
      },
      "region": {},
      "ptm": {},
      "processing": {},
      "variant": []
    },
    "metadata": {}
  }'
  expect_error(A3from_json(legacy_json), "index")
})

test_that("A3from_json rejects missing metadata", {
  json <- '{
    "$schema": "https://schema.rtemis.org/a3/v1/schema.json",
    "a3_version": "1.0.0",
    "sequence": "MKTAYIAKQRQISFVK",
    "annotations": {
      "site": {},
      "region": {},
      "ptm": {},
      "processing": {},
      "variant": []
    }
  }'
  expect_error(A3from_json(json), "metadata")
})

test_that("A3from_json reports duplicate positions with annotation path", {
  json <- '{
    "$schema": "https://schema.rtemis.org/a3/v1/schema.json",
    "a3_version": "1.0.0",
    "sequence": "MKTAYIAKQRQISFVK",
    "annotations": {
      "site": {},
      "region": {},
      "ptm": {
        "Phosphorylation": {
          "index": [7, 9, 7],
          "type": ""
        }
      },
      "processing": {},
      "variant": []
    },
    "metadata": {}
  }'
  expect_error(
    A3from_json(json),
    "annotations\\.ptm\\.Phosphorylation\\.index.*duplicate positions.*7"
  )
})

test_that("A3from_json accepts pre-parsed list", {
  lst <- list(
    `$schema` = "https://schema.rtemis.org/a3/v1/schema.json",
    a3_version = "1.0.0",
    sequence = "MKTAYIAKQRQISFVK",
    annotations = list(
      site = list(
        `Active site` = list(index = c(3L, 5L), type = "activeSite")
      ),
      region = list(),
      ptm = list(),
      processing = list(),
      variant = list()
    ),
    metadata = list(uniprot_id = "P12345")
  )
  x <- A3from_json(lst)
  expect_s7_class(x, A3)
  expect_identical(x@annotations@site$`Active site`@type, "activeSite")
  expect_identical(x@metadata@uniprot_id, "P12345")
})

test_that("read_A3json reports valid A3 version", {
  x <- create_A3(
    sequence = "MKTAYIAKQRQISFVK",
    site = list(
      `Active site` = annotation_position(c(3, 5), type = "activeSite")
    ),
    uniprot_id = "P12345"
  )
  filepath <- tempfile(fileext = ".json")
  write_A3json(x, filepath, overwrite = TRUE)

  expect_message(
    read_A3json(filepath, verbosity = 1L),
    "valid A3 1\\.0\\.0"
  )
})

# %% repr ----
repr <- new_generic("repr", "x")


# %% to_base ----
to_base <- new_generic("to_base", "x")


# %% to_json ----
to_json <- new_generic("to_json", "x")

# %% rtemis.core 0.0.4 ----

# %% clean_int ----
#' Clean integer input
#'
#' @details
#' The goal is to return an integer vector.
#' If the input is integer, it is returned as is.
#' If the input is numeric, it is coerced to integer only if the numeric values are integers,
#' otherwise an error is thrown.
#'
#' @param x Double or integer vector to check.
#'
#' @return Integer vector
#' @author EDG
#'
#' @keywords internal
#' @noRd
#'
#' @examples
#' clean_int(6L)
#' clean_int(3)
#' # clean_int(12.1) # Error
#' clean_int(c(3, 5, 7))
#' # clean_int(c(3, 5, 7.01)) # Error
clean_int <- function(x, xname = deparse(substitute(x))) {
  if (is.integer(x)) {
    return(x)
  } else if (is.numeric(x)) {
    if (all(x %% 1 == 0)) {
      storage.mode(x) <- "integer"
      return(x)
    } else {
      cli::cli_abort("{.var {xname}} must be integer.")
    }
  } else if (is.null(x)) {
    return(NULL)
  }
  cli::cli_abort("{.var {xname}} must be integer.")
}


# %% check_optional_scalar_character ----
#' Check Optional Scalar Character
#'
#' @param x Optional Character: Value to check.
#' @param arg_name Character: Argument name to use in error messages.
#'
#' @return Called for side effects.
#'
#' @author EDG
#' @keywords internal
#' @noRd
#'
#' @examples
#' check_optional_scalar_character(NULL, "my_arg") # Passes
#' check_optional_scalar_character("hello", "my_arg") # Passes
#' # Throw error:
#' try(check_optional_scalar_character(c("hello", "world"), "my_arg"))
#' try(check_optional_scalar_character(123, "my_arg"))
check_optional_scalar_character <- function(
  x,
  arg_name = deparse(substitute(x))
) {
  check_character(x, allow_null = TRUE, arg_name = arg_name)
  if (!is.null(x) && length(x) != 1L) {
    cli::cli_abort(
      "{.var {arg_name}} must be NULL or a single string."
    )
  }
  invisible()
}


#' Check Scalar Character
#'
#' @param x Character: Value to check.
#' @param arg_name Character: Argument name to use in error messages.
#'
#' @return Called for side effects.
#'
#' @author EDG
#' @keywords internal
#' @noRd
#'
#' @examples
#' check_scalar_character("hello", "my_arg") # Passes
#' # Throw error:
#' try(check_scalar_character(c("hello", "world"), "my_arg"))
#' try(check_scalar_character(123, "my_arg"))
check_scalar_character <- function(x, arg_name) {
  check_character(x, allow_null = FALSE, arg_name = arg_name)
  if (length(x) != 1L) {
    cli::cli_abort("{.var {arg_name}} must be a single string.")
  }
  invisible()
}

# %% clinvar_variants ----
#' Fetch ClinVar variants for a gene and parse them into A3Variant objects
#'
#' Queries the NCBI ClinVar database via E-utilities for all variants
#' associated with a gene, and returns those with protein-level amino acid
#' changes as a named list of `A3Variant` objects.
#'
#' Variants without a parseable protein change (intronic, regulatory, etc.)
#' are silently skipped.
#'
#' @param gene Character scalar: HGNC gene symbol, e.g. `"MAPT"`.
#' @param significance Character vector: Clinical significance filter. Any
#'   combination of `"pathogenic"`, `"likely_pathogenic"`,
#'   `"uncertain_significance"`, `"likely_benign"`, `"benign"`. Pass `NULL`
#'   to fetch all records regardless of significance.
#' @param max_variants Integer scalar: Maximum number of ClinVar records to
#'   fetch after filtering. Defaults to `500L`.
#' @param esearch_url Character scalar: NCBI E-utilities esearch endpoint.
#' @param esummary_url Character scalar: NCBI E-utilities esummary endpoint.
#' @param batch_size Integer scalar: Number of UIDs per esummary request.
#'   NCBI recommends no more than 500 per request.
#' @param verbosity Integer scalar: Verbosity level.
#'
#' @return Named list of `A3Variant` objects. Names use `{from}{position}{to}`
#'   notation (e.g. `"R406W"`), with a numeric suffix to disambiguate
#'   duplicates.
#'
#' @author EDG
#' @export
#'
#' @examples
#' # Requires internet connection and fetches data from ClinVar.
#' \dontrun{
#' # Pathogenic and likely pathogenic variants (default)
#' mapt_variants <- clinvar_variants("MAPT")
#' # All variants regardless of significance
#' mapt_all <- clinvar_variants("MAPT", significance = NULL)
#' }
clinvar_variants <- function(
  gene,
  significance = c("pathogenic", "likely_pathogenic"),
  max_variants = 500L,
  esearch_url = "https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi",
  esummary_url = "https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi",
  batch_size = 500L,
  verbosity = 1L
) {
  check_dependencies(c("httr", "jsonlite"))
  check_scalar_character(gene)
  check_character(significance)
  max_variants <- clean_int(max_variants)
  check_scalar_character(esearch_url)
  check_scalar_character(esummary_url)
  batch_size <- clean_int(batch_size)

  # -- Build search term ----
  valid_sig <- c(
    "pathogenic",
    "likely_pathogenic",
    "uncertain_significance",
    "likely_benign",
    "benign"
  )
  if (!is.null(significance)) {
    significance <- match.arg(significance, valid_sig, several.ok = TRUE)
    sig_filter <- paste(
      paste0('"', significance, '"', "[clinsig]"),
      collapse = " OR "
    )
    term <- paste0(gene, "[gene_name] AND (", sig_filter, ")")
  } else {
    term <- paste0(gene, "[gene_name]")
  }

  # -- Search: get ClinVar UIDs for this gene ----
  search_resp <- httr::GET(
    esearch_url,
    query = list(
      db = "clinvar",
      term = term,
      retmax = as.character(max_variants),
      retmode = "json"
    )
  )
  httr::stop_for_status(search_resp)
  search_dat <- jsonlite::fromJSON(
    httr::content(search_resp, as = "text", encoding = "UTF-8"),
    simplifyVector = TRUE
  )

  uids <- search_dat[["esearchresult"]][["idlist"]]
  if (length(uids) == 0L) {
    msg(
      "No ClinVar records found for gene:",
      highlight(gene),
      verbosity = verbosity
    )
    return(list())
  }
  msg(
    "Found",
    highlight(length(uids)),
    "ClinVar records for gene:",
    highlight(gene),
    verbosity = verbosity
  )

  # -- Summarize: fetch variant details in batches ----
  batches <- split(uids, ceiling(seq_along(uids) / batch_size))
  summaries <- lapply(batches, function(batch) {
    # POST avoids HTTP 414 (URI Too Long) when batches contain many UIDs
    resp <- httr::POST(
      esummary_url,
      body = list(
        db = "clinvar",
        id = paste(batch, collapse = ","),
        retmode = "json"
      ),
      encode = "form"
    )
    httr::stop_for_status(resp)
    dat <- jsonlite::fromJSON(
      httr::content(resp, as = "text", encoding = "UTF-8"),
      simplifyVector = FALSE
    )
    # Remove the "uids" index entry; keep only variant records
    result <- dat[["result"]]
    result[names(result) != "uids"]
  })
  # Flatten batches into a single named list keyed by UID
  all_summaries <- do.call(c, summaries)

  # -- Parse protein change: "{from}{position}{to}" e.g. "R406W" ----
  # Returns list(from, position, to) or NULL if unparseable.
  parse_protein_change <- function(pc) {
    m <- regmatches(pc, regexpr("^([A-Z])([0-9]+)([A-Z*])$", pc))
    if (length(m) == 0L) {
      return(NULL)
    }
    list(
      from = substr(m, 1L, 1L),
      position = as.integer(substr(m, 2L, nchar(m) - 1L)),
      to = substr(m, nchar(m), nchar(m))
    )
  }

  # -- Build A3Variant objects ----
  variant_list <- list()

  for (v in all_summaries) {
    pc_raw <- v[["protein_change"]]
    if (is.null(pc_raw) || !nzchar(pc_raw)) {
      next
    }

    # protein_change may contain multiple isoform changes separated by "/"
    pc_entries <- strsplit(pc_raw, "/", fixed = TRUE)[[1L]]
    # Use the first parseable entry
    parsed <- NULL
    for (entry in pc_entries) {
      parsed <- parse_protein_change(trimws(entry))
      if (!is.null(parsed)) break
    }
    if (is.null(parsed)) {
      next
    }

    # Clinical significance
    cs <- v[["clinical_significance"]]
    sig <- if (!is.null(cs[["description"]])) cs[["description"]] else ""
    rev <- if (!is.null(cs[["review_status"]])) cs[["review_status"]] else ""

    # Condition names
    traits <- v[["trait_set"]]
    conditions <- if (!is.null(traits) && length(traits) > 0L) {
      cnames <- vapply(
        traits,
        function(t) {
          nm <- t[["trait_name"]]
          if (!is.null(nm) && nzchar(nm)) nm else ""
        },
        character(1L)
      )
      paste(cnames[nzchar(cnames)], collapse = "; ")
    } else {
      ""
    }

    info <- list(
      from = parsed[["from"]],
      to = parsed[["to"]],
      clinicalSignificance = sig,
      reviewStatus = rev,
      conditions = conditions,
      accession = if (!is.null(v[["accession"]])) v[["accession"]] else "",
      clinvarId = if (!is.null(v[["uid"]])) v[["uid"]] else ""
    )

    variant_list <- c(
      variant_list,
      list(A3Variant(
        position = A3Position(data = parsed[["position"]]),
        info = info
      ))
    )
  }

  if (length(variant_list) == 0L) {
    msg("No protein-level variants parsed.", verbosity = verbosity)
    return(list())
  }

  # -- Name variants: "{from}{position}{to}", disambiguate duplicates ----
  raw_names <- vapply(
    variant_list,
    function(v) {
      paste0(v@info[["from"]], v@position@data, v@info[["to"]])
    },
    character(1L)
  )
  counts <- table(raw_names)
  running <- integer(length(counts))
  names(running) <- names(counts)
  final_names <- vapply(
    raw_names,
    function(nm) {
      if (counts[[nm]] == 1L) {
        nm
      } else {
        running[[nm]] <<- running[[nm]] + 1L
        paste0(nm, ".", running[[nm]])
      }
    },
    character(1L)
  )
  names(variant_list) <- final_names

  msg(
    "Parsed",
    highlight(length(variant_list)),
    "protein-level variants.",
    verbosity = verbosity
  )

  variant_list
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gene2sequence.R
\name{gene2sequence}
\alias{gene2sequence}
\title{Get the coding sequence of a gene}
\usage{
gene2sequence(
  gene,
  organism = "hsapiens",
  biomart = "ensembl",
  host = "https://www.ensembl.org",
  verbosity = 1L
)
}
\arguments{
\item{gene}{Character vector: One or more HGNC gene symbols.}

\item{organism}{Character scalar: Organism short name (Ensembl convention,
e.g. \code{"hsapiens"}).}

\item{biomart}{Character scalar: BioMart name.}

\item{host}{Character scalar: Host address.}

\item{verbosity}{Integer: Verbosity level.}
}
\value{
data.frame with columns "gene", "ensembl_transcript_id" and "sequence".
}
\description{
Get the coding sequence of a gene
}
\examples{
# Requires internet connection and fetches data from Ensembl using biomaRt.
\dontrun{
  mapt_seq <- gene2sequence("MAPT")
}
}
\author{
EDG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_uniprot.R
\name{uniprot_sequence}
\alias{uniprot_sequence}
\title{Fetch a protein sequence from UniProt}
\usage{
uniprot_sequence(
  accession,
  base_url = "https://rest.uniprot.org/uniprotkb",
  verbosity = 1L
)
}
\arguments{
\item{accession}{Character scalar: UniProt accession, e.g. \code{"P10636"}.}

\item{base_url}{Character scalar: UniProt REST API base URL.}

\item{verbosity}{Integer scalar: Verbosity level.}
}
\value{
Character scalar: amino acid sequence in single-letter code.
}
\description{
Lightweight FASTA-only fetch. Use this when you only need the amino acid
sequence. For full annotations and metadata use \code{\link[=uniprot_to_A3]{uniprot_to_A3()}}.
}
\examples{
# Requires internet connection and fetches data from UniProt.
\dontrun{
  seq <- uniprot_sequence("P10636")
}
}
\author{
EDG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a3.R
\name{annotation_variant}
\alias{annotation_variant}
\title{Create variant annotations for A3}
\usage{
annotation_variant(x, info = list())
}
\arguments{
\item{x}{Integer scalar: Position of the variant (1-based indexing).}

\item{info}{Named list: Additional information about the variant (e.g. reference and alternate
amino acids, variant type, etc.)}
}
\value{
A3Variant object
}
\description{
Creates an \code{A3Variant} object for variant annotations with position and info.
}
\examples{
# Create a variant annotation for position 10 with info about the variant
annotation_variant(10L, info = list(ref = "A", alt = "T", type = "missense"))
}
\author{
EDG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_a3.R
\name{get_alphafold}
\alias{get_alphafold}
\title{Get AlphaFold info for a given UniProt ID}
\usage{
get_alphafold(uniprotid)
}
\arguments{
\item{uniprotid}{Character: UniProt ID.}
}
\value{
data frame with AlphaFold info.
}
\description{
Get AlphaFold info for a given UniProt ID
}
\examples{
# Requires internet connection and fetches data from AlphaFold.
\dontrun{
get_alphafold("P10636")
}
}
\author{
EDG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a3.R
\name{concat}
\alias{concat}
\title{Concatenate character vector to single string for sequence input}
\usage{
concat(x)
}
\arguments{
\item{x}{Character vector to concatenate}
}
\value{
Single character string
}
\description{
Concatenate character vector to single string for sequence input
}
\examples{
# Concatenate a character vector of single-letter amino acids into a sequence string
concat(c("M", "A", "E", "P", "R"))
}
\author{
EDG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a3.R
\name{annotation_position}
\alias{annotation_position}
\title{Create position-based annotations for A3}
\usage{
annotation_position(x, type = NULL)
}
\arguments{
\item{x}{Integer vector: Positions of the annotation (1-based indexing).}

\item{type}{Optional character scalar: Annotation type. \code{NULL} is treated as
the empty string in the canonical A3 representation.}
}
\value{
Named list with \code{index} (A3Position) and \code{type} (character)
}
\description{
Creates an annotation spec (named list) with an \code{A3Position} index and optional type,
for use with \code{create_A3()}.
}
\examples{
# Create a site annotation for positions 5 and 17, of type "active site"
annotation_position(c(5L, 17L), type = "active site")
}
\author{
EDG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_uniprot.R
\name{uniprot_to_A3}
\alias{uniprot_to_A3}
\title{Fetch a UniProt entry and parse it into an A3 object}
\usage{
uniprot_to_A3(
  accession,
  base_url = "https://rest.uniprot.org/uniprotkb",
  verbosity = 1L
)
}
\arguments{
\item{accession}{Character scalar: UniProt accession, e.g. \code{"P10636"}.}

\item{base_url}{Character scalar: UniProt REST API base URL.}

\item{verbosity}{Integer scalar: Verbosity level.}
}
\value{
\code{A3} object with sequence, annotations, and UniProt metadata.
}
\description{
Fetches the UniProt JSON for the given accession via the UniProt REST API,
maps all sequence annotations to the A3 schema (see \code{specs/uniprot.md}), and
returns a fully populated \code{A3} object.
}
\details{
Feature types not mapped to A3 (mutagenesis, alternative sequences, sequence
conflicts, etc.) are silently ignored.
}
\examples{
# Requires internet connection and fetches data from UniProt.
\dontrun{
mapt <- uniprot_to_A3("P10636")
}
}
\author{
EDG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a3.R
\name{create_A3}
\alias{create_A3}
\title{Create an A3 object from sequence, annotations, and metadata}
\usage{
create_A3(
  sequence,
  site = list(),
  region = list(),
  ptm = list(),
  processing = list(),
  variant = list(),
  uniprot_id = NULL,
  description = NULL,
  reference = NULL,
  organism = NULL
)
}
\arguments{
\item{sequence}{Character scalar: Amino acid sequence string.}

\item{site}{Named list of site annotations}

\item{region}{Named list of region annotations}

\item{ptm}{Named list of PTM annotations}

\item{processing}{Named list of processing annotations}

\item{variant}{Named list of variant annotations}

\item{uniprot_id}{Optional character scalar: UniProt accession.}

\item{description}{Optional character scalar: Protein description.}

\item{reference}{Optional character scalar: Citation or URL.}

\item{organism}{Optional character scalar: Species name.}
}
\value{
A3 object
}
\description{
Create an A3 object from sequence, annotations, and metadata
}
\examples{
# Minimal: sequence only
a <- create_A3("MAEPRQEFEVMEDHAGTYGLGDRK")

# With site, region, and PTM annotations
a <- create_A3(
  "MAEPRQEFEVMEDHAGTYGLGDRK",
  site = list(
    Active_site = annotation_position(c(5L, 17L), type = "active site")
  ),
  region = list(
    Domain = annotation_range(rbind(c(3L, 10L), c(15L, 22L)),
      type = "repeat"
    )
  ),
  ptm = list(
    Phosphorylation = annotation_position(c(2L, 18L), type = "phosphoserine")
  ),
  uniprot_id = "P10636",
  organism = "Homo sapiens"
)
}
\author{
EDG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a3.R
\name{read_A3json}
\alias{read_A3json}
\title{Read \code{A3} object from JSON file}
\usage{
read_A3json(filepath, verbosity = 1L)
}
\arguments{
\item{filepath}{Character: Path to JSON file.}

\item{verbosity}{Integer: if greater than 0, print messages.}
}
\value{
\code{A3} object.
}
\description{
Read \code{A3} object from JSON file
}
\examples{
a3 <- create_A3("MAEPRQEFEVMEDHAGTYGLGDRK", uniprot_id = "P10636")
tmp <- tempfile(fileext = ".json")
write_A3json(a3, tmp)
a3_read <- read_A3json(tmp, verbosity = 0L)
unlink(tmp)
}
\author{
EDG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a3-package.R
\docType{package}
\name{rtemis.a3-package}
\alias{rtemis.a3}
\alias{rtemis.a3-package}
\title{rtemis.a3: Amino Acid Annotation format}
\description{
Amino Acid Annotation format utilities
}
\details{
\pkg{rtemis.a3}: Amino Acid Annotation format
}
\seealso{
Useful links:
\itemize{
  \item \url{https://a3.rtemis.org}
  \item Report bugs at \url{https://github.com/rtemis-org/a3/issues}
}

}
\author{
\strong{Maintainer}: E.D. Gennatas \email{gennatas@gmail.com} (\href{https://orcid.org/0000-0001-9280-3609}{ORCID}) [copyright holder]

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a3.R
\name{write_A3json}
\alias{write_A3json}
\title{Write \code{A3} object to JSON file}
\usage{
write_A3json(x, filepath, overwrite = FALSE)
}
\arguments{
\item{x}{\code{A3} object.}

\item{filepath}{Character: Path to save JSON file.}

\item{overwrite}{Logical: If TRUE, overwrite existing file.}
}
\value{
Invisible \code{x}. Writes JSON file as side effect.
}
\description{
Write \code{A3} object to JSON file
}
\examples{
a3 <- create_A3("MAEPRQEFEVMEDHAGTYGLGDRK", uniprot_id = "P10636")
tmp <- tempfile(fileext = ".json")
write_A3json(a3, tmp)
a3_read <- read_A3json(tmp)
unlink(tmp)
}
\author{
EDG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_pdb.R
\name{pdb_annotations}
\alias{pdb_annotations}
\title{Fetch PDB structural annotations for a UniProt accession}
\usage{
pdb_annotations(
  accession,
  pdb_id = NULL,
  pdbe_graph_url = "https://www.ebi.ac.uk/pdbe/graph-api",
  pdbe_api_url = "https://www.ebi.ac.uk/pdbe/api",
  verbosity = 1L
)
}
\arguments{
\item{accession}{Character scalar: UniProt accession, e.g. \code{"P10636"}.}

\item{pdb_id}{Optional character scalar: Four-character PDB ID (e.g. \code{"2mz7"}). If
\code{NULL}, the top-ranked structure from PDBe is used.}

\item{pdbe_graph_url}{Character scalar: PDBe Graph API base URL.}

\item{pdbe_api_url}{Character scalar: PDBe REST API base URL.}

\item{verbosity}{Integer scalar: Verbosity level.}
}
\value{
Named list with two elements:
\describe{
\item{\code{region}}{Named list of \code{A3Region} objects for secondary structure
(types: \code{"helix"}, \code{"betaStrand"}, \code{"turn"}).}
\item{\code{site}}{Named list of \code{A3Site} objects for ligand binding sites
(type: \code{"bindingSite"}).}
}
}
\description{
Queries the PDBe APIs for secondary structure and ligand binding sites from
experimental PDB structures, converts residue numbers to UniProt canonical
coordinates via SIFTS, and returns named lists of \code{A3Region} and \code{A3Site}
objects ready for use with \code{create_A3()}.
}
\details{
Structure ranking and SIFTS residue mappings come from the PDBe Graph API
\code{best_structures} endpoint. Secondary structure and binding sites are fetched
from the PDBe REST API.

When \code{pdb_id} is \code{NULL} the top-ranked structure (by PDBe coverage score) is
used automatically.
}
\examples{
# Requires internet connection and fetches data from PDBe.
\dontrun{
  ann  <- pdb_annotations("P10636")
  mapt <- create_A3(
    sequence = uniprot_sequence("P10636"),
    region   = ann[["region"]],
    site     = ann[["site"]]
  )
}
}
\author{
EDG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_clinvar.R
\name{clinvar_variants}
\alias{clinvar_variants}
\title{Fetch ClinVar variants for a gene and parse them into A3Variant objects}
\usage{
clinvar_variants(
  gene,
  significance = c("pathogenic", "likely_pathogenic"),
  max_variants = 500L,
  esearch_url = "https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi",
  esummary_url = "https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi",
  batch_size = 500L,
  verbosity = 1L
)
}
\arguments{
\item{gene}{Character scalar: HGNC gene symbol, e.g. \code{"MAPT"}.}

\item{significance}{Character vector: Clinical significance filter. Any
combination of \code{"pathogenic"}, \code{"likely_pathogenic"},
\code{"uncertain_significance"}, \code{"likely_benign"}, \code{"benign"}. Pass \code{NULL}
to fetch all records regardless of significance.}

\item{max_variants}{Integer scalar: Maximum number of ClinVar records to
fetch after filtering. Defaults to \code{500L}.}

\item{esearch_url}{Character scalar: NCBI E-utilities esearch endpoint.}

\item{esummary_url}{Character scalar: NCBI E-utilities esummary endpoint.}

\item{batch_size}{Integer scalar: Number of UIDs per esummary request.
NCBI recommends no more than 500 per request.}

\item{verbosity}{Integer scalar: Verbosity level.}
}
\value{
Named list of \code{A3Variant} objects. Names use \verb{\{from\}\{position\}\{to\}}
notation (e.g. \code{"R406W"}), with a numeric suffix to disambiguate
duplicates.
}
\description{
Queries the NCBI ClinVar database via E-utilities for all variants
associated with a gene, and returns those with protein-level amino acid
changes as a named list of \code{A3Variant} objects.
}
\details{
Variants without a parseable protein change (intronic, regulatory, etc.)
are silently skipped.
}
\examples{
# Requires internet connection and fetches data from ClinVar.
\dontrun{
# Pathogenic and likely pathogenic variants (default)
mapt_variants <- clinvar_variants("MAPT")
# All variants regardless of significance
mapt_all <- clinvar_variants("MAPT", significance = NULL)
}
}
\author{
EDG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/a3.R
\name{annotation_range}
\alias{annotation_range}
\title{Create range-based annotations for A3}
\usage{
annotation_range(x, type = NULL)
}
\arguments{
\item{x}{Integer matrix with 2 columns corresponding to start and end positions of the
annotation (1-based indexing).}

\item{type}{Optional character scalar: Annotation type. \code{NULL} is treated as
the empty string in the canonical A3 representation.}
}
\value{
Named list with \code{index} (A3Range) and \code{type} (character)
}
\description{
Creates an annotation spec (named list) with an \code{A3Range} index and optional type,
for use with \code{create_A3()}.
}
\examples{
# Create a region annotation for ranges 3-10 and 15-22, of type "repeat"
annotation_range(rbind(c(3L, 10L), c(15L, 22L)), type = "repeat")
}
\author{
EDG
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_a3.R
\name{aa_sub}
\alias{aa_sub}
\title{Perform amino acid substitutions}
\usage{
aa_sub(x, substitutions, verbosity = 1L)
}
\arguments{
\item{x}{Character vector: Amino acid sequence. e.g. \code{"ARND"} or
\code{c("A", "R", "N", "D")}.}

\item{substitutions}{Character vector: Substitutions to perform in the format
"OriginalPositionNew", e.g. \code{c("C291A", "C322A")}.}

\item{verbosity}{Integer: Verbosity level.}
}
\value{
Character vector with substitutions performed.
}
\description{
Perform amino acid substitutions
}
\examples{
aa_sub(c("A", "R", "N", "D"), c("R2K", "N3S"))
}
\author{
EDG
}
